define([
    "dojo/_base/declare",
    "thornberry/app/_Module",
    "thornberry/app/Component/LazyLoadView",
    "thornberry/app/Component/BillingManagement/Dashboard",
    "thornberry/app/Component/BillingManagement/UnbilledClaimsView",
    "thornberry/app/Component/BillingManagement/BilledClaimsView",
    "thornberry/app/Component/BillingManagement/ClaimFilesView"
],function(
    declare,
    _Module,
    LazyLoadView,
    Dashboard,
    UnbilledClaimsView,
    BilledClaimsView,
    ClaimFilesView
) {
    return declare(_Module,{
        title: "Billing Management",

        initViews: function(page) {
            var viewMap = {}
            var props = {
                props: {
                    onSetViewFilters: function(viewName,filters) {
                        var view = viewMap[viewName];
                        if (view) {
                            view.instance.set({
                                props: {
                                    filters: filters
                                }
                            })
                        }
                    },
                    onUpdateData: function() {
                        for (var viewName in viewMap) {
                            viewMap[viewName].instance.refreshData();
                        }
                    }
                }
            };

            viewMap["Dashboard"]        = new LazyLoadView({ viewClass: Dashboard,              title: "Dashboard",             props: props });
            viewMap["UnbilledClaims"]   = new LazyLoadView({ viewClass: UnbilledClaimsView,     title: "Unbilled Claims",       props: props,   preload: true });
            viewMap["BilledClaims"]     = new LazyLoadView({ viewClass: BilledClaimsView,       title: "Billed Claims",         props: props,   preload: true });
            viewMap["ClaimFiles"]       = new LazyLoadView({ viewClass: ClaimFilesView,         title: "Claim Files",           props: props,   preload: true });
           
            page.pushViewContainers([
                viewMap["Dashboard"],
                viewMap["UnbilledClaims"],
                viewMap["BilledClaims"],
                viewMap["ClaimFiles"]
            ]);
        }
    });
});