define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "thornberry/app/component/Report/View",
    "thornberry/app/utils",
    "thornberry/serverRequest"
],function(
    declare,
    lang,
    arrayUtils,
    ReportView,
    utils,
    serverRequest
) {
    return declare(ReportView,{
        title: "Billing Charges",
        showPresetControl: false,

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                reportClass: "Billing.Report.BillingCharges"
            });
        },

        getPageProps: function() {
            var t = this;
            var selectedIds = t.props.selectedIds;
            var canDelete = selectedIds.length > 0 && arrayUtils.every(selectedIds,function(id) {
                var rowData = t.getGridData(id);
                return rowData.BillingChargeType === "Medication" && rowData.ChargeClaimStatusInternal !== "Billed";
            });
            var canEdit = canDelete && selectedIds.length === 1;
            var canView = selectedIds.length === 1 && t.getGridData(selectedIds[0]).BillingChargeType === "Medication";

            return lang.mixin(t.inherited(arguments),{
                actions: [
                    { label: "Refresh",             onClick: lang.hitch(t,"refreshData") },
                    [
                        { label: "Add",                 onClick: function() {t.loadSubView("addEditCharge",{ objectId: "" })} },
                        { label: "Edit",                onClick: function() {t.loadSubView("addEditCharge",{ objectId: selectedIds[0] })},    hidden: !canEdit },
                        { label: "View",                onClick: function() {t.loadSubView("addEditCharge",{ objectId: selectedIds[0], mode: "view", viewOnly: !canEdit })},    hidden: !canView },
                        { label: "Delete",              onClick: lang.hitch(t,"deleteObject"), hidden: !canDelete },
                        { label: "Import Med Charges",  onClick: function() {t.loadSubView("medicationImport")} }
                    ]
                ]
            });
        },

        onRowDoubleClick: function(id) {
            
        },

        onShow: function() {
            var t = this;
            t.inherited(arguments);

            if (t.firstShow) {
                t.addSubView("addEditCharge","thornberry/app/Component/BillingCharges/AddEditChargeView",function() {
                    return {
                        mode: "edit", 
                        viewOnly: false,
                        onSave: lang.hitch(t,"refreshData")
                    };
                });
                t.addSubView("medicationImport","thornberry/app/Component/CSVImportWorkflow",function() {
                    return {
                        title: "Medication Charges",
                        cacheClass: "Billing.Charge.Medication",
                        onComplete: lang.hitch(t,"refreshData")
                    };
                });
            }
        },

        refreshData: function() {
            var t = this;
            if (t.controller) {
                // Force the summarization grid to update
                t.controller.modified.filters = true;
            }
            t.inherited(arguments);
        },

        deleteObject: function() {
            var t = this;
            utils.showMessage("Are you sure you want to delete all selected charges?",[
                { label: "Yes", callback: function() {
                    serverRequest("Billing.Charge.Medication","Delete",t.props.selectedIds).then(function() {
                        t.refreshData();
                    },utils.showMessage);
                } },
                { label: "No"}
            ]);
        }
    });
});