define([
    "dojo/_base/declare",
    "dojo/_base/array",
    "thornberry/app/Component/_Component",
    "thornberry/app/Component/_ViewMixin",
    "thornberry/app/Component/_DataFormMixin",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dojo/text!./Templates/ClaimFileDetailsView.html",
    "dojo/_base/lang",
    "thornberry/app/utils",
    "thornberry/report/formatter",
    "thornberry/app/service/BillingManagement",
    "dijit/Menu", 
    "dijit/MenuItem", 
    "dijit/form/DropDownButton"
],function(
    declare,
    arrayUtils,
    _Component,
    _ViewMixin,
    _DataFormMixin,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    template,
    lang,
    utils,
    formatter,
    service,
    Menu,
    MenuItem,
    DropDownButton
) {
    var structure = [
        { id: "patient", field: "PatientNameID", name: "Patient" },
        { id: "billPeriod", field: "Bill Period", name: "Bill Period" },
        { id: "insurance", field: "Insurance", name: "Insurance" },
        { id: "type", field: "Type", name: "Type" },
        { id: "gross", field: "GrossRevenue", name: "Gross", formatter: formatter.getForColumn({field:"GrossRevenue"},{formatter:"dollars"}) },
        { id: "net", field: "NetRevenue", name: "Net", formatter: formatter.getForColumn({field:"NetRevenue"},{formatter:"dollars"}) },
        { id: "claimId", field: "Claim ID", name: "Claim ID" }
    ];

    return declare([_Component,_ViewMixin,_DataFormMixin,_TemplatedMixin,_WidgetsInTemplateMixin], {
        templateString: template,

        value: null,

        getTitle: function() {
            var t = this;
            return "Details - "+t.props.value.fileName+" - "+t.props.value.form;
        },

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                value: null,
                onUpdateData: null
            });
        },

        getPageProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                content: t.domNode,
                actions : [
                    { label: "Back", onClick: t.props.onCloseView },
                    { label: "Download", onClick: lang.hitch(t,"download") },
                    { label: "Regenerate File", onClick: function() { t.loadSubView("postingSummary"); } },
                    t.markAsButton
                ]
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            t.grid = utils.createGrid(structure,t.gridContainer);

            var claimStatuses = [
                { id: 1, name: "Submitted" },
                { id: 2, name: "Accepted" },
                { id: 3, name: "Rejected" },
                { id: 4, name: "Replaced" }
            ];

            var menu = new Menu({ style: "display: none;"});
            arrayUtils.forEach(claimStatuses,function(item) {
                var menuItem = new MenuItem({
                    label: item.name,
                    onClick: function() { t.updateClaimFileStatus(item.id); }
                });
                menu.addChild(menuItem);
            });

            t.markAsButton = new DropDownButton({
                label: "Mark as...",
                dropDown: menu
            });
            t.markAsButton.startup();
        },
 
        refreshRendering: function() {
            var t = this;
            if (t.isShown) {
                if (t.props.value) {
                    t.props.value.records = t.props.value.claims.length;
                }
                t.set("value",t.props.value);

                t.grid.model.store.setData(t.props.value.claims);
                t.grid.model.clearCache();
                t.grid.body.refresh();
            }
            t.inherited(arguments);
        },

        onShow: function() {
            var t = this;
            t.resize();
            if (t.firstShow) {
                t.addSubView("postingSummary","thornberry/app/Component/BillingManagement/PostingSummaryView",function() {
                    var claimFileId = t.props.value.id;
                    return {
                        claimFileId: claimFileId,
                        agingDate: null,
                        onComplete: function() {
                            service.getClaimFileDetails(claimFileId).then(function(result) {
                                t.set("props",{ value: result });
                            },utils.showMessage);
                        }
                    }
                });
            }
        },

        download: function() {
            var t = this;
            service.getClaimFileDownloadLink(t.props.value.id).then(function(fileLinkId) {
                utils.promptDownload(fileLinkId);
            },utils.showMessage);
        },

        updateClaimFileStatus: function(status) {
            var t = this;
            var claimFileId = t.props.value.id;
            service.updateClaimFileStatus([claimFileId],status).then(function() {
                service.getClaimFileDetails(claimFileId).then(function(result) {
                    t.set("props",{ value: result });
                },utils.showMessage);
                if (t.props.onUpdateData) {
                    t.props.onUpdateData();
                }
            },utils.showMessage);
        },

        resize: function() {
            this.grid.resize();
        }
    });
});