define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "thornberry/app/Component/Report/View",
    "dijit/Menu", 
    "dijit/MenuItem", 
    "dijit/form/DropDownButton",
    "thornberry/app/service/BillingManagement",
    "thornberry/app/utils"
],function(
    declare,
    lang,
    arrayUtils,
    ReportView,
    Menu,
    MenuItem,
    DropDownButton,
    service,
    utils
) {
    return declare(ReportView,{
        title: "Claim Files",

        defaultToFilterView: false,
        showPresetControl: false,

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                reportClass: "Billing.Report.ClaimFiles",
                filters: null
            });
        },

        getPageProps: function() {
            var t = this;
            var selectedIds = t.props.selectedIds;
            return lang.mixin(t.inherited(arguments),{
                actions: [
                    { label: "Refresh", onClick: function() {t.controller.gridView.updateGridData(true);} },
                    [
                        { label: "View", onClick: function() { t.showDetails(t.props.selectedIds[0]); }, hidden: selectedIds.length !== 1 },
                        { label: "Download", onClick: lang.hitch(t,"download"), hidden: selectedIds.length !== 1 },
                        { label: "Regenerate File", onClick: function() { t.loadSubView("postingSummary"); }, hidden: selectedIds.length !== 1 },
                        (t.props.selectedIds.length > 0 ? t.markAsButton : null)
                    ]
                ]
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            var claimStatuses = [
                { id: 1, name: "Submitted" },
                { id: 2, name: "Accepted" },
                { id: 3, name: "Rejected" },
                { id: 4, name: "Replaced" }
            ];

            var menu = new Menu({ style: "display: none;"});
            arrayUtils.forEach(claimStatuses,function(item) {
                var menuItem = new MenuItem({
                    label: item.name,
                    onClick: function() { t.updateClaimFileStatus(item.id); }
                });
                menu.addChild(menuItem);
            });

            t.markAsButton = new DropDownButton({
                label: "Mark as...",
                dropDown: menu
            });
            t.markAsButton.startup();
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);

            // Force the report initialization to occur when the module preloads the view
            t.invalidateRendering();
        },

        onShow: function() {
            var t = this;
            t.inherited(arguments);
            if (t.firstShow) {
                t.addSubView("claimFileDetails","thornberry/app/Component/BillingManagement/ClaimFileDetailsView",function() {
                    return {
                        onUpdateData: t.props.onUpdateData
                    };
                });
                t.addSubView("postingSummary","thornberry/app/Component/BillingManagement/PostingSummaryView",function() {
                    return {
                        claimFileId: t.props.selectedIds[0],
                        agingDate: null,
                        onComplete: t.props.onUpdateData
                    }
                });
            }
        },

        onRowDoubleClick: function(id) {
            this.showDetails(id);
        },

        updateClaimFileStatus: function(status) {
            var t = this;
            var promise = service.updateClaimFileStatus(t.props.selectedIds,status);
            t.props.onSetLoadingPromise(promise);
            promise.then(t.props.onUpdateData,utils.showMessage);
        },

        download: function() {
            var t = this;
            service.getClaimFileDownloadLink(t.props.selectedIds[0]).then(function(fileLinkId) {
                utils.promptDownload(fileLinkId);
            },utils.showMessage);
        },

        showDetails: function(claimFileId) {
            var t = this;
            var promise = service.getClaimFileDetails(claimFileId).then(function(result) {
                t.loadSubView("claimFileDetails",{
                    value: result
                });
            },utils.showMessage);
            t.props.onSetLoadingPromise(promise);
        }
    });
});