define([
    "dojo/_base/declare",
    "thornberry/app/Component/_EditorComponent",
    "thornberry/app/Component/_TableFieldEditorMixin",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "thornberry/app/utils",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/when",
    "dojo/aspect",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dojo/text!./Templates/CodeEditor.html",
    "dojo/store/Memory",
    "dijit/form/ValidationTextBox",
    "dijit/form/RadioButton",
    "dijit/form/DateTextBox",
    "dijit/form/Form",
    "dijit/form/FilteringSelect",
    "dijit/form/Select",
    "dijit/form/NumberSpinner"
],function(
    declare,
    _EditorComponent,
    _TableFieldEditorMixin,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    utils,
    lang,
    arrayUtils,
    when,
    aspect,
    domConstruct,
    domStyle,
    template,
    Memory,
    ValidationTextBox
) {
    return declare([_EditorComponent,_TemplatedMixin,_WidgetsInTemplateMixin,_TableFieldEditorMixin], {
        templateString: template,

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                mode: "add",
                value: null,
                disabled: true,
                onCancel: null,
                onApply: null
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            t.addValidation("fromDate",utils.validators.dateBefore("toDate","The from date must be on or before the to date."));
            t.addValidation("toDate",utils.validators.dateAfter("fromDate","The to date must be on or after the from date."));

            t.addDisabled(null,function(name,value) {
                return name !== "type" && !value["type"];
            });
            t.addDisabled("fromDate",function(name,value) {
                return value["type"] != 1 && value["type"] != 2;
            });
            t.addDisabled("toDate",function(name,value) {
                return value["type"] != 2;
            });
            t.addDisabled("value",function(name,value) {
                return value["type"] != 3 && value["type"] != 5;
            });

            t.addRequirement(null,true);
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);

            // Clear fields when they become disabled automatically
            arrayUtils.forEach(t.fields,function(field) {
                t.own(aspect.after(field,"_setDisabledAttr",function(value) {
                    if (value) {
                        t.setFormValue(field.name,null,true);
                    }
                },true));
            });
        },

        postscript: function() {
            var t = this;
            t.inherited(arguments);
            t.set("value",t.props.value);
        },

        _onCancel: function() {
            var t = this;
            if (t.props.onCancel) {
                t.props.onCancel();
            }
        },

        _onApply: function() {
            var t = this;
            if (t.props.onApply) {
                t.props.onApply();
            }
        },

        _onReset: function() {
            var t = this;
            t.invalidateRendering();
        },

        onChangeModified: function(value) {
            var t = this;
            t.applyButton.set("disabled",!value);
            t.resetButton.set("disabled",!value);
        }
    });
});