define([
    "dojo/_base/declare",
    "gridx/Grid",
    "gridx/core/model/cache/Sync",
    "gridx/modules/SingleSort",
    "gridx/modules/select/Row",
    "gridx/modules/extendedSelect/Row",
    "gridx/modules/IndirectSelectColumn",
    "dojo/store/Memory",
    "thornberry/app/utils",
    "dojo/_base/array",
    "dojo/dom-style"
],function(
    declare,
    Grid,
    Cache,
    SingleSort,
    SelectRow,
    ExtendedSelectRow,
    IndirectSelectColumn,
    Memory,
    utils,
    arrayUtils,
    domStyle
) {
    var gridRowClickHandler = function(event) {
        // Summary:
        //      When connected with the grid's onRowClick event, this causes clicking the checkbox to add the row to those already selected, and clicking anywhere else
        //      to result in only the clicked row being selected
        if (event.columnIndex !== 0 && this.select.row.enabled) {
            var selected = this.select.row.getSelected();
            if (selected.length === 1 && selected[0] === event.rowId) {
                this.select.row.deselectById(event.rowId);
            } else {
                if (!event.ctrlKey) {
                    this.select.row.clear();
                }
                this.select.row.selectById(event.rowId);
            }
        }
    };

    return declare([Grid],{
        selectableRows: false,

        typeTable: null,

        cacheClass: null,
        store: null,
        structure: null,
        autoHeight: true,
        modules: null,
        style: "margin:8px",

        constructor: function() {
            this.cacheClass = Cache;
        },

        postMixInProperties: function() {
            var t = this;
            t.inherited(arguments);

            if (!t.store) {
                t.store = new Memory();
            }

            t.modules = [SingleSort];
            if (t.selectableRows) {
                t.modules.push(SelectRow);
                t.modules.push(ExtendedSelectRow);
                t.modules.push(IndirectSelectColumn);
            }

            t.structure = [
                { id: "type", field: "type", name: "Type", formatter: utils.formatters.tableItem("type",t.typeTable) },
                { id: "codeQualifier", field: "codeQualifier", name: "Code" },
                { id: "fromDate", field: "fromDate", name: "Date", formatter: utils.formatters.date("fromDate") },
                { id: "toDate", field: "toDate", name: "End", formatter: utils.formatters.date("toDate") },
                { id: "value", field: "value", name: "Value" }
            ];
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);

            if (t.select) {
                t.connect(t,"onRowClick",gridRowClickHandler);

                t.connect(t,"onCellDblClick",function(evt) {
                    t.onRowDoubleClick(evt.rowId);
                });
            }

            t.connect(t.body, "onAfterRow", function(row) {
                var item = row.grid.model.byId(row.id).item;
                try {
                    if (item.isFromClaimMapping) {
                        domStyle.set(row.node(),"color","gray");
                    }
                } catch (e) {console.error(e);}
            });
        },

        onRowDoubleClick: function(id) {

        },

        getSelected: function() {
            var t = this;
            var selectedIds = [];
            var rows = t.rows();
            for (var i = 0; i < rows.length; i++) {
                if (t.select.row.isSelected(rows[i].id)) {
                    selectedIds.push(rows[i].id);
                }
            }
            return selectedIds;
        },

        getAll: function() {
            return arrayUtils.map(this.rows(),function(row) {
                return row.id;
            });
        },
    });
});