define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "thornberry/app/Component/Report/View",
    "thornberry/report/GridView",
    "thornberry/app/Service/BillingManagement",
    "thornberry/app/utils"
],function(
    declare,
    lang,
    arrayUtils,
    ReportView,
    GridView,
    service,
    utils
) {
    return declare(ReportView,{
        title: "Unbilled Claims",

        defaultToFilterView: false,
        showPresetControl: false,

        controllerProps: {
            createGrid: function() {
                var t = this;
                t.set("gridView",new GridView({
                    controller: t,
                    title: t.report.displayName
                }));
            }
        },

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                reportClass: "Billing.Report.UnbilledClaims",
                filters: null
            });
        },

        getPageProps: function() {
            var t = this;
            var selectedIds = t.props.selectedIds;
            var canCode = selectedIds.length === 1 && t.getGridData(selectedIds[0]).Status !== "Pending";
            var allReadyToBill = selectedIds.length && arrayUtils.every(selectedIds,function(id) {
                var data = t.getGridData(id); 
                return data.Status === "Ready to Bill" && data.ContainsError; 
            });
            var allReadyToPost = selectedIds.length && arrayUtils.every(selectedIds,function(id) {
                return t.getGridData(id).Status === "Ready to Bill"; 
            });
            var allNotReadyToBill = selectedIds.length && arrayUtils.every(selectedIds,function(id) { return t.getGridData(id).Status === "Not Ready to Bill"; });
            return lang.mixin(t.inherited(arguments),{
                actions: [
                    { label: "Refresh", onClick: function() {t.controller.gridView.updateGridData(true);} },
                    { label: "Recalculate", onClick: function() {t.recalculateClaims();}, hidden: selectedIds.length === 0 },
                    [
                        { label: "View", onClick: function() { t.loadSubView("claim",{ mode: "view" }); }, hidden: selectedIds.length !== 1 },
                        { label: "Code", onClick: function() { t.loadSubView("claim",{ mode: "edit" }); }, hidden: !canCode  },
                        { label: "Post to A/R", onClick: function() { t.loadSubView("postToAr"); }, hidden: !allReadyToPost },
                        { label: "Make Ready to Bill", onClick: function() { t.toggleReadyToBill(true); }, hidden: !allNotReadyToBill },
                        { label: "Make Not Ready to Bill", onClick: function() { t.toggleReadyToBill(false); }, hidden: !allReadyToBill }
                    ]
                ]
            });
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);

            // Force the report initialization to occur when the module preloads the view
            t.invalidateRendering();
        },

        onShow: function() {
            var t = this;
            t.inherited(arguments);
            if (t.firstShow) {
                t.addSubView("claim","thornberry/app/Component/BillingManagement/ClaimView",function() {
                    return {
                        objectId: t.props.selectedIds[0],
                        viewOnly: (t.props.selectedIds.length && t.getGridData(t.props.selectedIds[0]).Status === "Pending"),
                        onSave: lang.hitch(t,"refreshData")
                    };
                });
                t.addSubView("postToAr","thornberry/app/Component/BillingManagement/PostToArView",function() {
                    return {
                        claimIds: t.props.selectedIds,
                        onComplete: t.props.onUpdateData
                    };
                });
            }
        },

        onRowDoubleClick: function(id) {
            this.loadSubView("claim",{ objectId: id, mode: "view", viewOnly: this.getGridData(id).Status === "Pending" } );
        },

        recalculateClaims: function() {
            var t = this;
            var promise = service.recalculateClaims(t.props.selectedIds).then(null,utils.showMessage);
            t.props.onSetLoadingPromise(promise);
            promise.then(function() {
                t.props.onUpdateData();
            },utils.showMessage)
        },
        
        toggleReadyToBill: function(readyToBill) {
            var t = this;
            var promise = service.toggleReadyToBill(t.props.selectedIds,readyToBill);
            t.props.onSetLoadingPromise(promise);
            promise.then(function() {
                t.props.onUpdateData();
            },utils.showMessage);
        }
    });
});