define([
    "dojo/_base/declare",
    "thornberry/app/Component/_Component",
    "thornberry/app/Component/_ViewMixin",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dojo/text!./Templates/ProcessResponseView.html",
    "dojo/text!./Templates/ProcessResponseItem.html",
    "dojo/_base/lang",
    "dojo/dom-style",
    "thornberry/app/utils",
    "thornberry/app/Service/InsuranceEligibility",
    "dojo/_base/array",
    "dijit/form/CheckBox"
],function(
    declare,
    _Component,
    _ViewMixin,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    template,
    itemTemplate,
    lang,
    domStyle,
    utils,
    eligibilityService,
    arrayUtils
) {
    var Item = declare([_Component,_ViewMixin,_TemplatedMixin,_WidgetsInTemplateMixin],{
        templateString: itemTemplate,

        details: "",
        checked: "",
        deleted: false,

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                item: {},
                number: 0,
                onProcess: null
            });
        },

        postMixInProperties: function() {
            var t = this;
            t.inherited(arguments);

            t.details = t.props.item.error ? t.props.item.htmlError : t.props.item.htmlData;
            t.checked = t.props.item.error ? "" : "checked";
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);
            domStyle.set(t.divider,"display",t.props.number > 1 ? "" : "none");
            domStyle.set(t.detailNode,"color",t.props.item.error ? "red" : "black");
        },

        _clickDelete: function() {
            this.set("deleted",true);
        },

        _clickUndo: function() {
            this.set("deleted",false);
        },

        _setDeletedAttr: function(value) {
            var t = this;
            t.deleted = value;
            domStyle.set(t.detailContainer,"display",value ? "none" : "");
            domStyle.set(t.deleteLink,"display",value ? "none" : "");
            domStyle.set(t.undoLink,"display",!value ? "none" : "");
            t.processCheck.set("disabled",value);
            if (value) {
                t.processCheck.set("checked",false);
            }
        }
    });

    return declare([_Component,_ViewMixin,_TemplatedMixin], {
        templateString: template,

        // properties
        title: "Review Eligibility Response(s)",

        policyIds: null,
        policyData: null,

        items: null,

        constructor: function() {
            this.items = [];
        },

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                policyIds: []
            });
        },

        getPageProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                content: t.domNode,
                actions : [
                    { label: "Back", onClick: t.props.onCloseView },
                    { label: "Process", onClick: lang.hitch(t,"process") }
                ]
            });
        },
 
        refreshRendering: function() {
            var t = this;
            t.set("policyIds",t.props.policyIds);
            t.inherited(arguments);
        },

        process: function() {
            var t = this;
            var responseIds = [];
            var deletedIds = [];
            arrayUtils.forEach(t.items,function(item) {
                if (item.processCheck.get("checked")) {
                    responseIds.push(item.props.item.responseId);
                } else if (item.deleted) {
                    deletedIds.push(item.props.item.responseId);
                }
            });
            eligibilityService.processResponses(responseIds,deletedIds).then(function() {
                if (t.props.onProcess) {
                    t.props.onProcess();
                }
                t.props.onCloseView();
            },utils.showMessage);
        },

        _setPolicyIdsAttr: function(policyIds) {
            var t = this;
            if (t.policyIds !== policyIds) {
                t.policyIds = policyIds;
                if (policyIds && policyIds.length) {
                    t.addReadyPromise(eligibilityService.getResponses(policyIds).then(function(policyData) {
                        t.set("policyData",policyData);
                    },utils.showMessage));
                } else {
                    t.set("policyData",[]);
                }
            }
        },

        _setPolicyDataAttr: function(policyData) {
            var t = this;
            arrayUtils.forEach(t.items,function(item) {
                item.destroy();
            });
            t.items = [];
            arrayUtils.forEach(policyData,function(item,i) {
                var itemWidget = new Item({ props: { item: item, number: i + 1 } });
                t.itemContainer.appendChild(itemWidget.domNode);
                t.items.push(itemWidget);
            });
        }
    });
});