define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "thornberry/app/component/Report/View",
    "thornberry/app/Service/Patient",
    "thornberry/app/Service/Settings",
    "thornberry/app/Service/PatientInsurance",
    "thornberry/app/Service/EnsembleFactory",
    "thornberry/app/utils",
    "dijit/Menu", 
    "dijit/MenuItem", 
    "dijit/form/ComboButton",
    "dojo/dom-style"
],function(
    declare,
    lang,
    arrayUtils,
    ReportView,
    patientService,
    settingsService,
    patientInsuranceService,
    ensembleFactory,
    utils,
    Menu,
    MenuItem,
    ComboButton,
    domStyle
) {
    return declare(ReportView,{
        title: "Insurance Eligibility",

        settings: null,

        // internals
        _requestComboButton: null,

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                reportClass: "Billing.Report.Eligibility"
            });
        },

        getPageProps: function() {
            var t = this;
            var selectedIds = t.props.selectedIds;
            var cannotProcess = selectedIds.length === 0 || arrayUtils.some(selectedIds,function(id) {
                return t.getGridData(id)["EligibilityResponseProcessed"] !== "No";
            });

            return lang.mixin(t.inherited(arguments),{
                actions: [
                    { label: "Refresh",            onClick: lang.hitch(t,"refreshData") },
                    { label: "Upload Response",     onClick: function() { t.loadSubView("uploadFileView"); } },

                    [
                        { label: "View",                                               onClick: function() { t.viewPolicy(t.props.selectedIds[0]); },                          hidden: selectedIds.length !== 1 },
                        (selectedIds.length ? t.settings.interfaceEnabled ? t._requestComboButton : { label: "Create File",         onClick: function() { t.createRequest() } } : null),
                        { label: "Process Response(s)", onClick: function() { t.loadSubView("processResponse",{ policyIds: selectedIds }); },   hidden: cannotProcess },
                    ]
                ]
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            var menu = new Menu({ style: "display: none;"});
            var menuItem = new MenuItem({
                label: "Create File",
                onClick: function() { t.createRequest() }
            });
            menu.addChild(menuItem);

            t._requestComboButton = new ComboButton({
                label: "Create Request",
                dropDown: menu,
                onClick: function() { t.createRequest(true) }
            });
            t._requestComboButton.startup();
        },

        refreshRendering: function() {
            var t = this;
            if (!t.settings) {
                t.addReadyPromise(settingsService.get("insuranceEligibility").then(function(settings) {
                    t.settings = settings;
                }));
            }
            t.inherited(arguments);
        },

        viewPolicy: function(id) {
            var t = this;
            ndocNum = t.getNDocNum(id);
            // Preload the header info so that there is no jagginess
            patientService.getHeaderInfo(ndocNum).then(function() {
                t.loadSubView("policyView",{ 
                    ndocNum: ndocNum,
                    objectId: id
                });
            });
        },

        onRowDoubleClick: function(id) {
            this.viewPolicy(id);
        },

        createRequest: function(useInterface) {
            var t = this;
            var service = ensembleFactory("Billing.Eligibility.EnsembleRequest");
            var promise = service({policyIds:t.props.selectedIds,useInterface:!!useInterface});
            promise.then(function(result) {
                t.loadSubView("requestSummary",result);
                t.refreshData();
            },utils.showMessage);
            t.props.onSetLoadingPromise(promise);
        },

        onShow: function() {
            var t = this;
            t.inherited(arguments);

            if (t.firstShow) {
                t.addSubView("policyView","thornberry/app/Component/PatientInsurance/AddEditPolicyView",function() {
                    return { 
                        viewOnly: true
                    };
                },true);
                t.addSubView("requestSummary","thornberry/app/Component/InsuranceEligibility/RequestSummaryView",function() {
                    return {};
                });
                t.addSubView("uploadFileView","thornberry/app/Component/FileUploadView",function() {
                    return { 
                        title: "Upload Eligibility Response File",
                        serverClass: "Billing.Eligibility.Service",
                        serverMethod: "UploadFile",
                        onUpload: function(promise) {
                            t.getSubView("uploadFileView").props.onCloseView();
                            var service = ensembleFactory("Billing.Eligibility.EnsembleResponse");
                            var promise = service(promise)
                            promise.then(function(policyIds) {
                                t.loadSubView("processResponse",{
                                    policyIds: policyIds
                                });
                                t.refreshData();
                            },utils.showMessage);
                            t.props.onSetLoadingPromise(promise,"Processing...");
                        }
                    };
                });
                t.addSubView("processResponse","thornberry/app/Component/InsuranceEligibility/ProcessResponseView",function() {
                    return {
                        onProcess: function() {
                            t.refreshData();
                        }
                    };
                });
            }
        },

        onLoadController: function() {
            var t = this;
            var promise = t.inherited(arguments);
            domStyle.set(t.presetControlContainer,"text-align","center");
            domStyle.set(t.presetControlContainer,"width","300px");
            domStyle.set(t.presetControlContainer,"position","absolute");
            domStyle.set(t.presetControlContainer,"right","200px");
            return promise;
        },

        onRefreshData: function() {
            var t = this;
            // Force the data in the editor to refresh in case there are changes
            var policyView = t.getSubView("policyView");
            if (policyView) {
                patientInsuranceService.clearPolicyCache();
                policyView.objectId = ""; 
            }
        },
        
        getNDocNum: function(objectId) {
            return objectId ? objectId.toString().split("||")[0] : "";
        }
    });
});