define([
    "dojo/_base/declare",
    "thornberry/app/Component/_Component",
    "thornberry/app/Component/_ViewMixin",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dojo/text!./Templates/RequestSummaryView.html",
    "dojo/text!./Templates/RequestSummaryItem.html",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-style",
    "thornberry/app/utils",
    "thornberry/app/Service/InsuranceEligibility",
    "thornberry/app/Service/Settings",
    "gridx/Grid",
    "gridx/core/model/cache/Sync",
    "dojo/store/Memory"
],function(
    declare,
    _Component,
    _ViewMixin,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    template,
    itemTemplate,
    lang,
    arrayUtils,
    domStyle,
    utils,
    insuranceEligibilityService,
    settingsService,
    Grid,
    Cache,
    Memory
    
) {
    var structure = [
        { id: "patient", field: "patient", name: "Patient" },
        { id: "insurance", field: "insurance", name: "Insurance" },
        { id: "policyNum", field: "policyNum", name: "Policy Number" },
        { id: "effectiveDate", field: "effectiveDate", name: "Effective Date", width: "90px" }    
    ];

    var createGrid = function(data,node) {
        var grid = new Grid({
            cacheClass: Cache,
            store: new Memory({data:data}),
            structure: structure,
            autoHeight: true
        });
        node.appendChild(grid.domNode);
        grid.startup();
        return grid;
    }

    var Item = declare([_Component,_ViewMixin,_TemplatedMixin],{
        templateString: itemTemplate,

        fileLabel: "",
        fileName: "",

        grid: null,

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                item: {
                    name: "",
                    policies: []
                },
                number: 0
            });
        },

        postMixInProperties: function() {
            var t = this;
            t.inherited(arguments);

            t.fileLabel = lang.replace("File #{number}",t.props);
            t.fileName = t.props.item.name;
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);
            
            domStyle.set(t.divider,"display",t.props.number > 1 ? "" : "none");

            t.grid = createGrid(t.props.item.policies,t.gridContainer);
        },

        resize: function() {
            this.grid.resize();
        }
    });

    return declare([_Component,_ViewMixin,_TemplatedMixin,_WidgetsInTemplateMixin], {
        templateString: template,

        // properties
        title: "Request Summary",

        settings: null,

        items: null,

        constructor: function() {
            this.items = [];
        },

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                useInterface: false,
                files: [],
                requestId: "",
                logId: "",
                fileLinkId: ""
            });
        },

        getPageProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                content: t.domNode,
                actions : [
                    { label: "Back", onClick: t.props.onCloseView },
                    { label: "Download", onClick: lang.hitch(t,"download") }
                ]
            });
        },
 
        refreshRendering: function() {
            var t = this;
            if (!t.settings) {
                t.addReadyPromise(settingsService.get("insuranceEligibility").then(function(settings) {
                    t.settings = settings;
                    if (settings.interfaceEnabled) {
                        t.interfaceMessageNode.innerText = lang.replace("These eligibiliy requests have been submitted via {clearinghouse}.",settings);
                    }
                }));
            }

            if (t.isShown) {
                domStyle.set(t.interfaceMessageContainer,"display",t.props.useInterface == 1 ? "" : "none");

                arrayUtils.forEach(t.items,function(widget) {widget.destroy()});
                t.items = [];
                arrayUtils.forEach(t.props.files,function(file,idx) {
                    var item = new Item({props: { item: file, number:idx+1 } });
                    t.items.push(item);
                    t.itemContainer.appendChild(item.domNode);
                });
            }
            t.inherited(arguments);
        },

        onShow: function() {
            var t = this;
            arrayUtils.forEach(t.items,function(widget) {widget.resize()});
        },

        download: function() {
            var t = this;
            utils.promptDownload(t.props.fileLinkId);
            insuranceEligibilityService.setRequestDownloaded(t.props.logId);
        }
    });
});