define([
    "dojo/_base/declare",
    "thornberry/app/Component/_Component",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dojo/text!./Templates/Page.html",
    "dojo/dom-style",
    "dojo/dom-construct",
    "dojo/on",
    "dojo/topic",
    "dojo/when",
    "dojo/keys",
    "dojo/Deferred",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/_base/fx",
    "dojo/json",
    "dojo/dom-geometry",
    "thornberry/app/Service/Patient",
    "thornberry/app/Service/Report",
    "thornberry/ReportManagerII",
    "thornberry/recyclable",
    "thornberry/app/utils",
    "thornberry/Toolbar",
    "dijit/registry",
    "dijit/form/Button",
    "dijit/form/DropDownButton",
    "dijit/DropDownMenu",
    "dijit/MenuItem",
    "dijit/layout/ContentPane",
    "dijit/layout/LayoutContainer",
    "thornberry/layout/WorkspaceTitlePane",
    "thornberry/account/HeaderPaneSmall",
    "dijit/form/TextBox",
    "dijit/ProgressBar"
],function(
    declare,
    _Component,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    template,
    domStyle,
    domConstruct,
    on,
    topic,
    when,
    keys,
    Deferred,
    lang,
    arrayUtils,
    fx,
    json,
    domGeom,
    patientService,
    reportService,
    ReportManager,
    recyclable,
    utils,
    Toolbar,
    registry,
    Button,
    DropDownButton,
    DropDownMenu,
    MenuItem
) {
    // TODO: in _findView() check each view at a given view level before popping to the next (onBeforeClose())
    // TODO: use create and use _renderData in RefreshRendering() to optimize

    var RButton = recyclable(declare(Button,{
        onRecycle: function() {
            domStyle.set(this.domNode,"display","none");
            domConstruct.place(this.domNode,"body");
            this.set("disabled",true);
        },

        onInitializeRecycled: function(props) {
            domStyle.set(this.domNode,"display","");
            this.set("label",props.label);
            this.set("disabled",props.disabled);
            this.onClick = props.onClick;
        }
    }));

    return declare([_Component,_TemplatedMixin,_WidgetsInTemplateMixin], {
        templateString: template,

        reportClass: "",

        _ready: null, // function to be called when the page is ready to show
        _firstLoad: true,
        _lastContent: null,
        _viewStack: null,
        _currentView: null,
        _waitForView: false, // true when waiting for the view to decide whether it can be closed or not
        _renderData: null,
        _viewMenuShown: false,
        _lastView: null,
        _lastBreadcrumbs: null,
        _lastViewsString: "",

        _lastNDocNum: "",
        _viewProps: null,
        _defaultIdx: null,
        _toolbarActionId: 0,

        getInitialProps: function() {
            var t = this;
            return {
                // Must always be set by the view's getPageProps() or they revert to defaults
                loadingIconShown:   false,
                searchButtonShown:  false,
                filterButtonShown:  false,
                sumButtonShown:     false,
                optionsButtonShown: false,
                printButtonShown:   false,
                helpButtonShown:    false,
        
                onSearch: function(text) {},
                onClickFilter: function() {},
                onClickSum: function() {},
                onClickOptions: function() {},
                onClickPrint: function() { t.setUpReportManager(true); },
                onClickHelp: function() {},
        
                accountSummaryShown: false,
        
                actions: null,          // [ { label: "Action", onClick: function() {} }, [ { label: "Action", onClick: function() {} } ... ], ... ]
                viewMenuShown: true,
                navigationActive: true,
        
                content: null,          // dom node

                // These carry forward
                ndocNum: "",
                quitLabel: "",
                onQuit: function() {},

                report: {
                    reportClass: "",
                    getContext: null
                }
            };
        },

        constructor: function() {
            var t = this;
            t._viewStack = [];
            t.addInvalidatingProperties(["_viewStack","_currentView"]);
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            t.viewMenu = new DropDownMenu();
            t.viewMenuButton = new DropDownButton({
                dropDown: t.viewMenu
            });
            t.viewMenuContainerNode.appendChild(t.viewMenuButton.domNode);

            t.toolbar = new Toolbar();
            domConstruct.place(t.toolbar.domNode,t.toolbarContainer);
            domStyle.set(t.toolbar.domNode,"display","inline-block");
            t.toolbar.startup();
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);
            t.setViewProps();
            t._initSearch();

            t.own(on(window,"resize",lang.hitch(t,"resizeToolbar")));
        },

        refreshRendering: function() {
            var t = this;

            if (!t.domNode || !t._currentView || !t.props.content) {
                return;
            }

            var resize = false;

            var title = t.props.viewTitle;
            var views = t.getViews();

            var breadcrumbs;
            if (t.props.view === t._lastView) {
                breadcrumbs = t._lastBreadcrumbs;
            } else {
                breadcrumbs = t.getBreadcrumbs();
                t._lastView = t.props.view;
                t._lastBreadcrumbs = breadcrumbs;
            }

            t.workspaceTitlePane.registeredOperations = {};
            t.workspaceTitlePane.set("title",title);
            var lastOperationId = 0;
            breadcrumbs = arrayUtils.map(breadcrumbs,function(breadcrumb) {
                var operationId = ++lastOperationId;
                if (t.props.navigationActive) {
                    t.workspaceTitlePane.registerOperation(operationId,breadcrumb.onClick);
                }
                return { text: breadcrumb.label, operation: operationId };
            });

            t.workspaceTitlePane.registerOperation("quit",t.props.onQuit);
            t.workspaceTitlePane.set("breadcrumbs",breadcrumbs);

            domStyle.set(t.loadingIconNode,"display",(t.props.loadingIconShown ? "" : "none"));
            domStyle.set(t.searchButtonNode,"display",(t.props.searchButtonShown ? "" : "none"));
            domStyle.set(t.filterButtonNode,"display",(t.props.filterButtonShown ? "" : "none"));
            domStyle.set(t.sumButtonNode,"display",(t.props.sumButtonShown ? "" : "none"));
            domStyle.set(t.optionsButtonNode,"display",(t.props.optionsButtonShown ? "" : "none"));
            domStyle.set(t.printButtonNode,"display",(t.props.printButtonShown ? "" : "none"));
            domStyle.set(t.helpButtonNode,"display",(t.props.helpButtonShown ? "" : "none"));

            var viewNodeShown = false;
            if (t.props.viewMenuShown && views.length > 1) {
                domStyle.set(t.viewMenuContainerNode,"display","");
                viewNodeShown = true;
                t._viewMenuShown = true;

                if (json.stringify(views) !== t._lastViewsString) {
                    t._lastViewsString = json.stringify(views);
                    t.viewMenu.destroyDescendants();
                    var selected = null;
                    arrayUtils.forEach(views,function(view) {
                        if (view.selected) {
                            selected = view;
                        }
                        t.viewMenu.addChild(new MenuItem({
                            label: view.label,
                            onClick: view.onClick
                        }));
                    });
                    t.viewMenuButton.set("label",(selected ? selected.label : "(Select)"));
                }
                t.viewMenuButton.set("disabled",!t.props.navigationActive);
                
            } else {
                domStyle.set(t.viewMenuContainerNode,"display","none");
                t._viewMenuShown = false;
            }

            if (viewNodeShown || t.props.loadingIconShown || t.props.searchButtonShown || t.props.filterButtonShown || t.props.sumButtonShown || t.props.optionsButtonShown || t.props.printButtonShow || t.props.helpButtonShown) {
                domStyle.set(t.actionContainerNode,"position","absolute");
            } else {
                domStyle.set(t.actionContainerNode,"position","");
            }

            t.fetchAccountHeaderData(t.props.ndocNum);
            var display = domStyle.get(t.accountSummaryPane.domNode,"display");
            if (display !== (t.props.accountSummaryShown ? "block" : "none")) {
                resize = true;
            }
            domStyle.set(t.accountSummaryPane.domNode,"display",(t.props.accountSummaryShown ? "" : "none"));

            arrayUtils.forEach(registry.findWidgets(t.actionContainer),function(widget) {
                if (widget.recycle) {
                    widget.recycle();
                } else {
                    domConstruct.place(widget.domNode,t.hiddenContentNode);
                }
            });
            domConstruct.empty(t.actionContainer);

            if (t.props.actions) {
                t._processActions(t.props.actions,t.actionContainer);
            }

            if (t._lastContent !== t.props.content) {
                if (t._lastContent) {
                    var oldNode = t.contentContainer.domNode.firstChild;
                    if (oldNode) {
                        domConstruct.place(oldNode,t.hiddenContentNode)
                    }
                    t._lastContent = null;
                }
                if (t.props.content) {
                    t.contentContainer.set("content",t.props.content);
                    t._lastContent = t.props.content;
                    t._currentView.onShow();
                }
            }

            if (resize || t._firstLoad) {
                t.layoutContainer.resize();
            }

            if (t._firstLoad) {
                t._firstLoad = false;
                if (t._ready) {
                    t.defer("_ready");
                    t.defer(function() {
                        topic.publish("page","ready");
                    });
                }
            }

            if (t.props.report) {
                t.setReport(t.props.report);
            }

            t.resizeToolbar();
        },

        _onSearch:          function(text) { if (this.props.onSearch && this.props.navigationActive)  { this.props.onSearch(text); } },
        _onClickFilter:     function() { if (this.props.onClickFilter && this.props.navigationActive)  { this.props.onClickFilter(); } },
        _onClickSum:        function() { if (this.props.onClickSum && this.props.navigationActive)     { this.props.onClickSum(); } },
        _onClickOptions:    function() { if (this.props.onClickOptions && this.props.navigationActive) { this.props.onClickOptions(); } },
        _onClickPrint:      function() { if (this.props.onClickPrint && this.props.navigationActive)   { this.props.onClickPrint(); } },
        _onClickHelp:       function() { if (this.props.onClickHelp && this.props.navigationActive)    { this.props.onClickHelp(); } },

        fetchAccountHeaderData: function(ndocNum) {
            var t = this;
            if (t._lastNDocNum !== ndocNum && t.props.accountSummaryShown) {
                t.accountSummaryPane.set("AccountHeaderData",null);
                t._lastNDocNum = ndocNum;
                patientService.getHeaderInfo(ndocNum).then(function(result) {
                    if (t.props.ndocNum === ndocNum) {
                        t.accountSummaryPane.set("AccountHeaderData",result);
                    }
                });
            }
        },

        _initSearch: function() {
			var t = this;
			on(t.searchInput,"blur",function() {
				setTimeout(function() {
					fx.animateProperty({
						node: t.searchInput.domNode,
						properties: {
							width: {
								start: 200,
								end: 0
							}
						},
						onEnd: function() {
							domStyle.set(t.searchInput.domNode,"display","none");
                            t.searchInput.set("value","");
                            if (t._viewMenuShown) {
                                domStyle.set(t.viewMenuContainerNode,"display","");
                            }
						}
					}).play();
				},100);
			});
			
			var searchClick = function() {
				if (domStyle.get(t.searchInput.domNode,"display") === "none") {
                    domStyle.set(t.searchInput.domNode,"display","");
                    if (t._viewMenuShown) {
                        domStyle.set(t.viewMenuContainerNode,"display","none");
                    }
					fx.animateProperty({
						node: t.searchInput.domNode,
						properties: {
							width: {
								start: 0,
								end: 200
							}
						}
					}).play();
					t.searchInput.focus();
				} else {
					var searchText = t.searchInput.get("value");
                    if (searchText != "") {
                        t._onSearch(searchText);
                    }
				}
            };
            
			t.own(
                on(t.searchButtonNode,"click",searchClick),
                on(t.searchInput,"keyDown",function(event) {
                    if (event.keyCode === keys.ENTER) {
                        searchClick();
                        t.searchInput.onBlur();
                    }
                })
            );
        },
        
        _processActions: function(actions,containerNode,level) {
            var t = this;
            if (typeof level === "undefined") {
                level = 0;
            }

            if (level === 0) {
                if (!lang.isArray(actions)) {
                    actions = [actions];
                } else {
                    actions = actions.slice(); // replace with a shallow copy so that we won't modify the original actions array
                }

                // Remove the last set of actions or actions in an action group and put in a toolbar to support dynamic sizing better. Only works if all are passed as labels
                var arr = actions;
                var toolbarActions = [];
                var toolbarDisables = [];

                var end = false;
                while (!end && arr.length > 0) {
                    var lastActionItem = arr[arr.length - 1];
                    if (!lastActionItem) {
                        arr.pop();
                    } else if (lang.isArray(lastActionItem)) {
                        // replace with a shallow copy so that we won't modify the original actions array
                        arr.push(arr.pop().slice());
                        arr = arr[arr.length - 1];
                    } else if (!lastActionItem.domNode) {
                        if (!lastActionItem.hidden) {
                            toolbarActions.push({
                                actionId: ++t._toolbarActionId,
                                type: "button",
                                label: lastActionItem.label,
                                onClick: lastActionItem.onClick
                            });
                            if (!t.props.navigationActive || lastActionItem.disabled) {
                                toolbarDisables.push(t._toolbarActionId);
                            }
                        }
                        arr.pop();
                    } else {
                        end = true;
                    }
                }
            }
            arrayUtils.forEach(actions,function(action,i) {
                if (action) {
                    if (lang.isArray(action)) {
                        var style = "display:inline-block;"+(i > 0 ? "margin-left:20px;" : "");
                        var groupContainer = domConstruct.toDom("<div style='"+style+"'></div>");
                        containerNode.appendChild(groupContainer);
                        t._processActions(action,groupContainer,level + 1);
                    } else if (action.domNode) {
                        // In case the action is an initialized form widget
                        containerNode.appendChild(action.domNode);
                        action.set("disabled",!t.props.navigationActive);
                    } else if (action && !action.hidden) {
                        var button = RButton.getRecycled( {
                            label: action.label,
                            onClick: action.onClick,
                            disabled: !t.props.navigationActive || (action.disabled ===  true)
                        });
                        containerNode.appendChild(button.domNode);
                    }
                }
            });

            if (level === 0) {
                domStyle.set(containerNode,"display",(containerNode.childNodes.length ? "inline-block" : "none"));

                t.toolbar.setActions(toolbarActions.reverse());
                arrayUtils.forEach(toolbarDisables,function(id) {
                    t.toolbar.enable(id,false);
                });
            }
        },

        getBreadcrumbs: function() {
            var t = this;
            var breadcrumbs = [];
            if (t.props.quitLabel) {
                breadcrumbs.push({label:t.props.quitLabel,onClick:function() {t._findView(-1)}});
            }
            if (t.props.title) {
                breadcrumbs.push({label:t.props.title,onClick:function() {
                    t._findView(0).then(function(stackLevel) {
                        if (stackLevel === 0 && t._viewStack[0].views.length > t._defaultIdx) {
                            t.switchView(t._defaultIdx);
                        }
                    });
                }});
            }
            var max = t._viewStack.length - 2;
            arrayUtils.forEach(t._viewStack,function(value,i) {
                if (i <= max) {
                    breadcrumbs.push({ label: value.views[value.idx].getTitle(), onClick: function() {t._findView(i)}});
                }
            });
            return breadcrumbs;
        },

        getViews: function() {
            var t = this;
            var value = t._viewStack[t._viewStack.length - 1];
            var views = [];
            arrayUtils.forEach(value.views,function(view,idx) {
                views.push({
                    label: view.getTitle(),
                    onClick: function() {
                        t.switchView(idx);
                    },
                    selected: (view === t._currentView)
                });
            });
            return views;
        },

        addView: function(view,show) {
            // Add a view at the current stack level
            var t = this;
            var stackLength = t._viewStack.length;
            if (stackLength === 0) {
                t.pushViewContainers(view);
            } else {
                var last = t._viewStack[stackLength-1];
                last.views.push(view);
                if (show) {
                    show = (t._currentView ? t._currentView.onBeforeHide() : true);
                    when (show,function(show) {
                        if (show) {
                            last.idx = last.views.length - 1;
                        }
                        t.set("_viewStack",t._viewStack);
                    });
                } else {
                    t.set("_viewStack",t._viewStack);
                }
            }
        },

        removeView: function(view) {
            // Remove a view from the current stack level
            var t = this;
            when (view.onBeforeClose,function(canClose) {
                if (canClose) {
                    view.onClose();
                    var stackLength = t._viewStack.length;
                    if (stackLength > 0) {
                        var last = t._viewStack[stackLength-1];
                        last.views = arrayUtils.filter(last.views,function(v,i) {
                            var remove = v === view;
                            if (remove) {
                                if (i === last.idx) {
                                    last.idx = 0;
                                }
                            }
                            return !remove;
                        });
                        if (last.views.length === 0) {
                            t._viewStack.pop();
                        }
                        t.set("_viewStack",t._viewStack);
                    }
                }
            });
        },

        switchView: function(idx) {
            // switch to another view at the current level
            var t = this;
            var last = t._viewStack[t._viewStack.length-1];
            if (last && last.idx !== idx) {
                when(t._currentView.onBeforeHide(),function(result) {
                    if (result) {
                        t._setViewIndex(idx);
                    }
                });
            }
        },

        pushViewContainers: function(views,idxToShow) {
            // views: one or more view containers. If multiple, then a select box is shown to switch between them
            // idxToShow: if multiple view are passed in, the index in the array of the one to show by default. The first is used if no value is set
            var t = this;
            if (views) {
                if (!lang.isArray(views)) {
                    views = [views];
                }
                idxToShow = idxToShow || 0;
                t._viewStack.push({ views: views, idx: idxToShow });
                t.set("_viewStack",t._viewStack);
            }
            if (!t._defaultIdx && t._viewStack.length === 1) {
                t._defaultIdx = idxToShow;
            }
        },

        quit: function() {
            // hook for outside elements to trigger quit action
            if (this.props.navigationActive) {
                this._findView(-1);
            }
        },

        popView: function() {
            var t = this;
            t._currentView.onClose();
            t._viewStack.pop();
            t.set("_viewStack",t._viewStack);
        },

        _findView: function(stackLevel) {
            // TODO: check each view (onBeforeClose()) at a given stack level before popping to the next

            // This method unwinds the view stack to the level passed in. 
            // It calls onBeforeClose() on each view to determine whether it can be closed.
            var t = this;
            if (t._waitForView) { return null; }

            var deferred = new Deferred();
            var stackLength = t._viewStack.length;
            if (stackLevel + 1 < stackLength) {
                var value = t._viewStack[t._viewStack.length - 1];

                // Create a copy of the views array with the current view sorted to the beginning
                var sortedViews = [];
                arrayUtils.forEach(value.views, function(view,i) {
                    (i == value.idx ? sortedViews.unshift(view) : sortedViews.push(view));
                });

                var view = sortedViews[0];
                t._waitForView = true;
                when(view.onBeforeClose(),function(value) {
                    t._waitForView = false;
                    if (value) {
                        if (stackLength > 1) {
                            t.popView();
                            var promise = t._findView(stackLevel);
                            if (promise) {
                                promise.then(function(result) {
                                    deferred.resolve(result);
                                });
                            } else {
                                deferred.resolve(stackLength - 1);
                            }
                        } else if (t.props.onQuit) {
                            view.onClose();
                            deferred.resolve(stackLength - 1);
                            t.props.onQuit();
                        }
                    } else {
                        deferred.resolve(stackLength - 1);
                    }
                });
            } else {
                deferred.resolve(stackLength - 1);
            }
            return deferred.promise;
        },

        _setViewIndex: function(idx) {
            var t = this;
            var last = t._viewStack[t._viewStack.length - 1];
            last.idx = idx;
            t.set("_viewStack",t._viewStack);
        },

        _set_viewStackAttr: function(viewStack) {
            var t = this;
            t._viewStack = viewStack;
            var last = viewStack[viewStack.length-1];
            var view = last.views[last.idx];
            if (view.declaredClass === "thornberry.app.Component.LazyLoadView") {
                // LazyLoadView integration
                view = view.load();
                last.views[last.idx] = view;
            }
            t.set("_currentView",view);
        },

        _set_currentViewAttr: function(view) {
            // Note: this does not update idx on the view stack item. This isn't to be triggered directly
            var t = this;
            
            view.onBeforeShow();
            view.set("props",t._viewProps);
            
            if (view && t._currentView !== view) {
                if (t._currentView) {
                    t._currentView.onHide();
                    t._currentView.set("props",{
                        onRequestClose: null,
                        onPushView: null,
                        onUpdatePageProps: null,
                        onSetNavigationActive: null,
                        onAddView: null,
                        onRemoveView: null
                    });
                }
            }
            t._currentView = view;
        },

        ready: function(ready) {
            var t = this;
            if (!t._firstLoad) {
                ready();
            } else {
                t._ready = ready;
            }
        },

        setViewProps: function() {
            var t = this;
            t._viewProps = {
                onCloseView: function() {
                    t._findView(t._viewStack.length - 2);
                },
                onPushView: lang.hitch(t,"pushViewContainers"),
                onUpdatePageProps: function(props,mixInExisting) {
                    when(props,function(props) {
                        var mixin = (mixInExisting ? t.props : t.getInitialProps());
                        t.set("props",lang.mixin(mixin,{
                            onQuit: t.props.onQuit,
                            quitLabel: t.props.quitLabel,
                            ndocNum: t.props.ndocNum
                        },props));
                    });
                },
                onResizeContainer: lang.hitch(t.contentContainer,"resize"),
                onAddView: lang.hitch(t,"addView"),
                onRemoveView: lang.hitch(t,"removeView"),
                onSwitchView: lang.hitch(t,"switchView"),
                onQuit: t.props.onQuit,
                onGenerateReport: lang.hitch(t,"generateReport"),
                onSetLoadingPromise: lang.hitch(t,"setLoadingPromise")
            };
        },

        setUpReportManager: function(show) {
            var t = this;
            var deferred = new Deferred();
            if (!t.reportManager) {
                t.reportManager = new ReportManager({
                    userSettings: generalUserSettings || null
                });
                t.reportManager.startup();
            }
            if (t.report) {
                t.reportPromise.then(function() {
                    var reportProps = t.props.report;
                    var getContext = reportProps.getContext;
                    if (!getContext) {
                        getContext = t.generateGetContext(reportProps.getFilters,reportProps.getOptions);
                    }

                    t.reportManager.set("report",t.report);
                    t.reportManager.getContext = getContext;

                    deferred.resolve();
                })
            } else {
                t.reportManager.set("report",null);
                deferred.resolve();
            }
            if (show) {
                t.reportManager.show();
            }
            return deferred.promise;
        },

        generateGetContext: function(getFilters,getOptions) {
            return function(presentationType,report) {
                return {
                    userId:         utils.environment.getUserId(),
                    deviceAppId:    utils.environment.getDeviceAppId(),
                    presentation:   presentationType,
                    reportClass:    report.className,
                    filters:        (getFilters ? getFilters(presentationType,report) : []),
                    options:        (getOptions ? getOptions(presentationType,report) : {}),
                };
            }
        },

        setReport: function(report) {
            var t = this;
            var reportClass = report.reportClass;
            if (reportClass != t.reportClass) {
                if (reportClass) {
                    t.reportPromise = reportService.getReport(reportClass).then(function(rep) {
                        t.report = rep;
                        t.reportClass = reportClass;
                    });
                } else {
                    t.report = null;
                    t.reportClass = "";
                }
            }
        },

        generateReport: function(report) {
            /*
                report: {
                    reportClass: "Report.Definition.*",     // the report class name. The corresponding report definition will be loaded from the server
                    presentation: "PDF"                     // optional, default is PDF
                    getFilters:                             // a function to return an array of report filters  - signature = getFilters(presentationType,report)
                    getOptions:                             // a function to return a map of report options     - signature = getOptions(presentationType,report)
                    getContext:                             // provided as a backup if getFilters and getOptions do not provide the required flexibility
                    childReport: false                      // optional - for use in special cases
                }
            */  

            var t = this;
            t.setUpReportManager().then(function() {
                reportService.getReport(report.reportClass).then(function(rep) {
                    var presentationType = report.presentation || "PDF";
                    if (report.childReport) {
                        rep.childReport = true;
                    }
                    var defaultOptions = t.reportManager.getDefaultOptions(rep);
                    if (report.getOptions) {
                        defaultOptions = lang.mixin(defaultOptions,report.getOptions(presentationType,rep))
                    }
                    var getContext = report.getContext;
                    if (!getContext) {
                        getContext = t.generateGetContext(report.getFilters,report.getOptions);
                    }
                    t.reportManager.getContext = getContext;
                    t.reportManager.generateReport(rep,presentationType,defaultOptions);
                });
            });
        },

        resizeToolbar: function() {
            var t = this;
            var RIGHT_RESERVE = (t._viewMenuShown ? 475 : 225);

            var maxWidth = domGeom.position(t.actionPane.domNode).w - RIGHT_RESERVE;
            var actionWidth = domGeom.position(t.actionContainer).w;
            var toolbarWidth = maxWidth - actionWidth - 16;
            toolbarWidth = (toolbarWidth < 50 ? 50 : toolbarWidth);
            domStyle.set(t.toolbar.domNode,"width",toolbarWidth+"px");

            t.toolbar.resize();

            domStyle.set(t.actionPane.domNode,"min-width",(actionWidth + RIGHT_RESERVE + 50)+"px");
            //t.toolbar.resize();
        },

        setLoadingPromise: function(promise,message) {
            var t = this;
            message = message ? message : "Loading...";
            t.loadMessageNode.innerHTML = message;
            domStyle.set(t.loaderNode,"display","");
            promise.then(null,null,function(message) {
                t.loadMessageNode.innerHTML = message;
            });
            promise.always(function() {
                domStyle.set(t.loaderNode,"display","none");
            });
        }
    });
})