define([
    "dojo/_base/declare",
    "thornberry/app/Component/_EditorComponent",
    "thornberry/app/Component/_TableFieldEditorMixin",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "thornberry/app/utils",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/when",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dojo/text!./Templates/AuthDetailEditor.html",
    "dojo/store/Memory",
    "thornberry/serverRequest",
    "thornberry/form/searchBox",
    "dijit/form/ValidationTextBox",
    "dijit/form/RadioButton",
    "dijit/form/DateTextBox",
    "dijit/form/Form",
    "dijit/form/FilteringSelect",
    "dijit/form/Select",
    "dijit/form/NumberSpinner"
],function(
    declare,
    _EditorComponent,
    _TableFieldEditorMixin,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    utils,
    lang,
    arrayUtils,
    when,
    domConstruct,
    domStyle,
    template,
    Memory,
    serverRequest,
    searchBox,
    ValidationTextBox
) {
    return declare([_EditorComponent,_TemplatedMixin,_WidgetsInTemplateMixin,_TableFieldEditorMixin], {
        templateString: template,

        supplyId: "",

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                mode: "add",
                value: {
                    type: "D",
                    basis: "V",
                    period: "W",
                    minFreqAuth: 0,
                    maxFreqAuth: 1,
                    duration: 1
                },
                disabled: true,
                onCancel: null,
                onApply: null
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            var supplySearch = new searchBox({
                name: "supply",
                maxLength: 75,
                allowFreeText: true,
                getSearchResults: function(text) {
                    return serverRequest("UI.Utility","SupplySearch",{searchText:text}).then(function(results) {
                        return new Memory({data:results});
                    });
                },
                _getValueAttr: function() {
                    return this.searchTextBox.get("value");
                },
                executeSearch: function() {
                    /* change from inherited: no escapeText() call on options */
                    var t = this;
                    if (ValidationTextBox.prototype.validator.call(t.searchTextBox,t.searchTextBox.get("value"),t.searchTextBox.constraints)) {
                        var searchString = t.searchTextBox.get("value");
                        if (searchString !== t._lastSearchString) {
                            t._lastSearchString = searchString;
                            
                            t.resultsSelect.domNode.length = 0;
                            t.resultsSelect.set("value","");
                            t.selectButton.set("disabled",true);
                            
                            if (searchString !== "") {
                                when(t.getSearchResults(searchString),function(store) {
                                    t.store = store;
                                    arrayUtils.forEach(store.data,function(item) {
                                        domConstruct.create("option",{label:(item.label || item.name),value:item.id},t.resultsSelect.domNode);
                                    });
                                });
                            }
                        }
                    }
                },
                onChange: function(value) {
                    if (lang.isArray(value)) {
                        value = value[0];
                    }
                    t.supplyId = (this.store.get(value) ? value : "");
                }
            },t.supplyNode);
            supplySearch.startup();

            // tables
            t.setTable("period",[
                { id: "D", name: "day" },
                { id: "W", name: "week" },
                { id: "M", name: "month" },
                { id: "Y", name: "need" }
            ]);
            t.addTableFilter("discipline",function(name,item) {
                return item.active === "Yes";
            });
            t.addTableFilter("disciplineServiceId",function(name,item) {
                return item.active === "Yes" &&  item.discipline == t.fieldMap["discipline"][0].get("value");
            });

            // validations
            t.addValidation("minFreqAuth",function(name,value,lastValues) {
                if (lastValues["maxFreqAuth"] && value > lastValues["maxFreqAuth"]) {
                    utils.showMessage("The minimum authorized must be less than or equal to the maximum authorized.");
                    return false;
                }
                return true;
            });
            t.addValidation("maxFreqAuth",function(name,value,lastValues) {
                if (lastValues["minFreqAuth"] && value < lastValues["minFreqAuth"]) {
                    utils.showMessage("The maximum authorized must be greater than or equal to the minimum authorized.");
                    return false;
                }
                return true;
            });

            // field logic
            t.addLogic("type",function(name,value) {
                domStyle.set(t.disciplineFieldContainer,"display",(value === "D" ? "" : "none"));
                domStyle.set(t.supplyFieldContainer,    "display",(value === "S" ? "" : "none"));
                domStyle.set(t.disciplineServiceIdFieldContainer,"display",(value === "D" ? "" : "none"));
                domStyle.set(t.supplyServiceIdFieldContainer,    "display",(value === "S" ? "" : "none"));
                domStyle.set(t.basisFieldContainer,    "display",(value === "D" ? "" : "none"));

                if (value === "D") {
                    t.setFormValue("supply","");
                    t.setFormValue("supplyServiceId","");
                } else if (value === "S") {
                    t.setFormValue("discipline","");
                    t.setFormValue("disciplineServiceId","");
                }
            },true);
            t.addLogic("discipline",function(name,value) {
                if (!value) {
                    t.setFormValue("disciplineServiceId","");
                }
                t.refreshTableField("disciplineServiceId","");
            });
            t.addLogic("period",function(name,value) {
                domStyle.set(t.durationContainer,"display",(value === "Y" ? "none" : ""));
                if (value === "Y") {
                    t.setFormValue("duration",1);
                }

                var periodItem = t.getTable("period").get(value);
                t.periodLabel.innerText = (periodItem ? periodItem.name+"(s)" : "");
            },true);
            t.addLogic(["basis","type"],function(name,value,lastValues,onSetValue) {
                if (!onSetValue) {
                    arrayUtils.forEach(["minFreqAuth","maxFreqAuth","quantityReq"],function(name) {
                        if (!t.fieldMap[name][0].isValid()) {
                            t.setFormValue(name,null);
                        }
                    });
                }
                var type = (name === "type" ? value : lastValues["type"]);
                var basis = (name === "basis" ? value : lastValues["basis"]);
                var quarters = (type === "D") && (basis === "H");

                t.fieldMap["maxFreqAuth"][0].constraints.min    = (quarters ? 0.25 : 1);
                arrayUtils.forEach(["minFreqAuth","maxFreqAuth","quantityReq"],function(field) {
                    t.fieldMap[field][0].constraints.places = (quarters ? "0,2" : 0);
                    t.fieldMap[field][0].validate();
                });
                if (quarters && onSetValue && name === "basis") {
                    // Reapply these values when the editor is loaded since rounding may be off
                    arrayUtils.forEach(["minFreqAuth","maxFreqAuth","quantityReq"],function(name) {
                        t.setFormValue(name,lastValues[name]);
                    });
                }
            },true);
            t.addLogic(["type"],function(name,value,lastValues) {
                var type = (name === "type" ? value : lastValues["type"]);
                var basis = (name === "basis" ? value : lastValues["basis"]);
                var quarters = (type === "D") && (basis === "H");

                t.fieldMap["maxFreqAuth"][0].constraints.min    = (quarters ? 0.25 : 1);
                arrayUtils.forEach(["minFreqAuth","maxFreqAuth","quantityReq"],function(field) {
                    t.fieldMap[field][0].constraints.places = (quarters ? "0,2" : 0);
                    t.fieldMap[field][0].validate();
                });
            },true);
            t.addLogic(["minFreqAuth","maxFreqAuth","quantityReq"],function(name,value,lastValues) {
                if (lastValues.basis === "H") {
                    var newValue = Math.round((value * 4)) / 4;
                    if (newValue != value) {
                        t.setFormValue(name,newValue);
                    }
                }
            });

            // disabled logic
            t.addDisabled("type",function() {
                return t.props.mode === "edit";
            });
            t.addDisabled("disciplineServiceId",function(name,value) {
                return !value.discipline;
            });

            // requirements
            t.addRequirement([
                "maxFreqAuth",
                "period"
            ],true);
            t.addRequirement(["discipline"],function(name,value) {return value.type === "D";});
            t.addRequirement(["duration"],function(name,value) {return value.period !== "Y";});
            //t.addRequirement(["supply"],    function(name,value) {return value.type === "S";});
        },

        postscript: function() {
            var t = this;
            t.inherited(arguments);
            t.set("value",t.props.value);
        },

        refreshRendering: function() {
            var t = this;
            t.inherited(arguments);
            
            if (t.props.value) {
                t.supplyId = t.props.value.supplyId;
            }
        },

        _onCancel: function() {
            var t = this;
            if (t.props.onCancel) {
                t.props.onCancel();
            }
        },

        _onApply: function() {
            var t = this;
            if (t.props.onApply) {
                t.props.onApply();
            }
        },

        _onReset: function() {
            var t = this;
            t.invalidateRendering();
        },

        onChangeModified: function(value) {
            var t = this;
            t.applyButton.set("disabled",!value);
            t.resetButton.set("disabled",!value);
        },

        _setValueAttr: function(value) {
            var t = this;
            var newVal = value;
            if (value) {
                newVal = lang.mixin({},value,{
                    discipline: (value.type === "D" ? value.disciplineSupply : ""),
                    supply:     (value.type === "S" ? value.disciplineSupply : ""),
                    disciplineServiceId: (value.type === "D" ? value.serviceId : ""),
                    supplyServiceId:     (value.type === "S" ? value.serviceId : ""),
                });
            }
            t.inherited(arguments,[newVal]);
        },

        _getValueAttr: function() {
            var t = this;
            var value = t.inherited(arguments);
            value.disciplineSupply  = (value.type === "D" ? value.discipline : value.supply);
            value.serviceId         = (value.type === "D" ? value.disciplineServiceId : value.supplyServiceId);
            value.basis             = (value.type === "D" ? value.basis : "");
            value.supplyId          = t.supplyId;
            delete value.discipline, value.supply, value.disciplineServiceId, value.serviceSupplyId;
            return value;
        }
    });
});