define([
    "dojo/_base/declare",
    "thornberry/app/Component/_EditorComponent",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "thornberry/app/Component/_TableFieldEditorMixin",
    "dojo/_base/lang",
    "dojo/on",
    "dojo/when",
    "dojo/dom-style",
    "dojo/dom-class",
    "dojo/query",
    "dojo/aspect",
    "dojo/text!./Templates/BillRateEditor.html",
    "thornberry/recyclable",
    "thornberry/serverRequest",
    "thornberry/app/utils",
    "dijit/form/ValidationTextBox",
    "dijit/form/RadioButton",
    "dijit/form/DateTextBox",
    "dijit/form/FilteringSelect",
    "dijit/form/Form",
    "dijit/Dialog"
],function(
    declare,
    _EditorComponent,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    _TableFieldEditorMixin,
    lang,
    on,
    when,
    domStyle,
    domClass,
    query,
    aspect,
    template,
    recyclable,
    serverRequest,
    utils
) {
    var BillRateEditor = declare([_EditorComponent,_TemplatedMixin,_WidgetsInTemplateMixin,_TableFieldEditorMixin], {
        templateString: template,

        recycled: false,

        // internals
        useGrossRate: false,
        chargeCode: "",
        effectiveFromDate: "",
        chargeCodeDetails: null,

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                data: null,
                onCopy: null,
                onRemove: null,
                useGrossRate: false,
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            var style = { width : "55px", "font-size" : "9pt" };
            domStyle.set(t.copyButton.titleNode,style);
            domStyle.set(t.removeButton.titleNode,style);

            t.addValidation("effectiveFromDate",utils.validators.dateBefore("effectiveToDate","The effective from date must be on or before the effective to date."));
            t.addValidation("effectiveToDate",utils.validators.dateAfter("effectiveFromDate","The effective to date must be on or after the effective from date."));

            t.addTableFilter("chargeCode",function(name,tableItem) {
                return tableItem.name !== "SIA";
            });
            t.addTableFilter("rateType",function(name,tableItem,value) {
                var chargeCode = t.getField("chargeCode").get("value");
                var chargeCodeObj = t.tableManager.get("Tables.ChargeCodes").get(chargeCode);
                if (chargeCodeObj) {
                    var chargeType = chargeCodeObj.ChargeType;
                    if (chargeType == 1) {
                        return tableItem.id != 6;
                    } else if (chargeType == 2) {
                        return tableItem.id == 6;  
                    } else if (chargeType == 3) {
                        return tableItem.id == 4 || tableItem.id == 6;
                    } else {
                        return tableItem.id == 1;
                    }
                }
                return true;
            });
            t.addTableFilter(["tier1RateType","tier2RateType"],function(name,tableItem) {
                return tableItem.id != 6;
            });

            t.addLogic("chargeCode",function(name,value,lastValues,onSet) {
                if (onSet) {
                    t.effectiveFromDate = t.value.effectiveFromDate;
                    t.chargeCode        = t.value.chargeCode;
                    t.chargeCodeDetails = t.props.value;
                }
            },true);

            t.addLogic(["chargeCode","effectiveFromDate","netRateType"],function(name,value,lastValues,onSet) {
                var value = (onSet ? t.props.value : t.get("value"));
                if ( value && value.chargeCode != "") {
                    when (t.tableManager.get("Tables.ChargeCodes"),function(store) {
                        var chargeCodeObj = store.get(value.chargeCode);
                        var chargeType = chargeCodeObj.ChargeType;
                        var onComplete = function() {
                            if (t.chargeCodeDetails) {
                                t.chargeCodeDetails.chargeType = chargeType;
                                t.setFormValue("grossAmount",t.chargeCodeDetails.grossAmount);

                                // Set rate type
                                var defaultRateType = t.getDefaultRateType(chargeType);
                                if (value.netRateType == 2) { // Percentage
                                    t.refreshTableField("rateType",t.chargeCodeDetails.rateType || defaultRateType);
                                    t.setFormValue("rateType",t.chargeCodeDetails.rateType || defaultRateType,true);
                                } else { // Fixed
                                    if (chargeType != 1 && chargeType != 3) {
                                        t.refreshTableField("rateType",defaultRateType);
                                        t.setFormValue("rateType",defaultRateType,true);
                                    } else {
                                        t.refreshTableField("rateType");
                                    }
                                }

                                // Set revenue code
                                if (chargeType == 4 || chargeType == 6) {
                                    t.setFormValue("revenueCode",(chargeType == 4 ? t.chargeCodeDetails.revenueCode : ""));
                                }

                                // Set HCPCS/CPT code and modifier
                                if (chargeType == 4 || chargeType == 5 || chargeType == 6) {
                                    t.setFormValue("hcpcsCptCode",(chargeType == 6 ? "" : t.chargeCodeDetails.hcpcsCptCode),true);
                                }
                                if (chargeType == 4 || chargeType == 5) {
                                    t.setFormValue("hcpcsCptModifiers",t.chargeCodeDetails.hcpcsCptModifiers);
                                }
                                
                                // Tiered rates
                                if (chargeType != 1) {
                                    t.setFormValue("useTieredRates",false,true);
                                }

                                t.invalidate("fieldState");
                            }
                        }

                        if (value.chargeCode != t.chargeCode || value.effectiveFromDate != t.effectiveFromDate || t.chargeCodeDetails === null) {
                            t.chargeCode = value.chargeCode;
                            t.effectiveFromDate = value.effectiveFromDate;
                            serverRequest("Patient.Insurance.Utility","GetChargeCodeDetails",{
                                chargeCode: value.chargeCode,
                                effectiveDate: value.effectiveFromDate
                            }).then(function(result) {
                                t.chargeCodeDetails = result;
                                onComplete();
                            });
                        } else {
                            onComplete();
                        }
                    });
                } else {
                    t.setFormValue("grossAmount","");
                    t.chargeCodeDetails = null;
                }
            },true);

            t.addLogic("rateType",function(name,value,lastValues) {
                if (value == 1 || value == 6) {
                    t.setFormValue("roundingType",2);
                }
            },true);

            t.addLogic("hcpcsCptCode",function(name,value) {
                if (value == "") {
                    t.setFormValue("hcpcsCptModifiers","");
                }
            },true);

            t.addLogic("useTieredRates",function(name,value) {
                t.tieredRateButton.set("disabled",!value.length || t.props.disabled);
                if (!value.length) {
                    t.setFormValue("",{
                        tier1NetRate: "",
                        tier1NetRateType: "",
                        tier1RateType: "",
                        tier1RevenueCode: "",
                        tier1HcpcsCptCode: "",
                        tier1HcpcsCptModifiers: "",
                        tier1MaxMinutes: "",
                        tier2NetRate: "",
                        tier2NetRateType: "",
                        tier2RateType: "",
                        tier2RevenueCode: "",
                        tier2HcpcsCptCode: "",
                        tier2HcpcsCptModifiers: ""
                    },true,true);
                }
            },true);

            t.addLogic(["tier1NetRateType","tier2NetRateType"],function(name,value) {
                var rateTypeFieldName = (name == "tier1NetRateType" ? "tier1RateType" : "tier2RateType");
                if (value == 2) {
                    t.setFormValue(rateTypeFieldName,t.chargeCodeDetails.rateType || 1,true);
                }
            });

            t.addRequirement([
                "chargeCode",
                "revenueCode",
                "tier1NetRate",
                "tier1RevenueCode",
                "tier1MaxMinutes",
                "tier2NetRate",
                "tier2RevenueCode"
            ],true);

            t.addDisabled("grossAmount",true);
            t.addDisabled(["netRate","netRateType"],function() {return t.useGrossRate});
            t.addDisabled("rateType",function(name,values) {return values.netRateType == 2 || (t.chargeCodeDetails && t.chargeCodeDetails.chargeType != 1 && t.chargeCodeDetails.chargeType != 3);});
            t.addDisabled("roundingType",function(name,values) {return values.rateType == 1 || values.rateType == 6;});
            t.addDisabled("revenueCode",function() {return t.chargeCodeDetails && (t.chargeCodeDetails.chargeType == 4 || t.chargeCodeDetails.chargeType == 6)});
            t.addDisabled("hcpcsCptCode",function() {return t.chargeCodeDetails && (t.chargeCodeDetails.chargeType == 4 || t.chargeCodeDetails.chargeType == 5 || t.chargeCodeDetails.chargeType == 6)});
            t.addDisabled("hcpcsCptModifiers",function() {return t.chargeCodeDetails && t.chargeCodeDetails.chargeType == 5 || t.getField("hcpcsCptCode").get("value") == ""});
            t.addDisabled("useTieredRates",function() {return !t.chargeCodeDetails || t.chargeCodeDetails.chargeType != 1});
            t.addDisabled(["tier1RateType","tier2RateType"],function(name,values) { return values[name == "tier1RateType" ? "tier1NetRateType" : "tier2NetRateType"] == 2;})
            t.addDisabled("tier1HcpcsCptModifiers",function() {return t.getField("tier1HcpcsCptCode").get("value") == ""});
            t.addDisabled("tier2HcpcsCptModifiers",function() {return t.getField("tier2HcpcsCptCode").get("value") == ""});
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);

            t.getField("chargeCode").queryExpr = '${0}*';

            t.own(
                on(t.tieredRateDialog,"cancel",function() {
                    t.tieredRateDialog.hide();
                }),
                on(t.tieredRateButton,"click",function() {
                    t.tieredRateDialog.show();
                }),
                on(t.closeButton,"click",function() {
                    t.tieredRateDialog.hide();
                }),
                aspect.after(t,"updateFieldState",function() {
                    domClass.toggle(t.tieredRateButtonContainer,"pinkBackground",t.getField("useTieredRates").get("checked") && query(".pinkBackground",t.tieredRateDialog.containerNode).length > 0);
                })
            );
        },

        refreshRendering: function() {
            var t = this;
            t.inherited(arguments);

            t.copyButton.set("disabled",t.props.disabled);
            t.removeButton.set("disabled",t.props.disabled);

            t.tieredRateButton.set("disabled",!t.props.value || !t.props.value.useTieredRates || t.props.disabled);

            t.set("useGrossRate",t.props.useGrossRate);
        },

        _onCopy: function() {
            var t = this;
            if (t.props.onCopy) {
                t.props.onCopy();
            }
        },

        _onRemove: function() {
            var t = this;
            if (t.props.onRemove) {
                t.props.onRemove();
            }
        },

        _setUseGrossRateAttr: function(value) {
            var t = this;
            if (t.useGrossRate !== value && value) {
                when (t._tableLoadPromise,function() {
                    t.setFormValue("netRate",100);
                    t.setFormValue("netRateType",2,true);
                });
            }
            t.useGrossRate = value;
        },
        
        getDefaultRateType: function(chargeType) {
            return {
                2: 6 // Facility - Per Day
            }[chargeType] || 1;  
        },

        onRecycle: function() {
            domStyle.set(this.domNode,"display","none");
            this.set("disabled","true");
            this.recycled = true;
        },

        onInitializeRecycled: function() {
            domStyle.set(this.domNode,"display","");
            //this.set("props",{ value: this.getInitialProps() });
            this.useGrossRate = false;
            this.recycled = false;
        },

        _initForm: function() {
            var t = this;
            t.inherited(arguments);
            var oldGetDescendantFormWidgets = lang.hitch(t.form,t.form._getDescendantFormWidgets);
            t.form._getDescendantFormWidgets = function() {
                var dialogWidgets = oldGetDescendantFormWidgets(t.tieredRateDialog.getChildren());
                return oldGetDescendantFormWidgets().concat(dialogWidgets);
            }
        }
    });
    return recyclable(BillRateEditor);
});