define([
    "dojo/_base/declare",
    "thornberry/app/Component/_Component",
    "thornberry/app/Component/_EditorViewMixin",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dojo/text!./Templates/GeneralView.html",
    "thornberry/app/utils",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/when",
    "gridx/Grid",
    "gridx/core/model/cache/Sync",
    "gridx/modules/SingleSort",
    "dojo/store/Memory",
    "thornberry/app/Service/PatientInsurance",
    "thornberry/app/Component/PatientInsurance/GeneralEditor",
    "gridx/core/model/extensions/FormatSort"
],function(
    declare,
    _Component,
    _EditorViewMixin,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    template,
    utils,
    lang,
    arrayUtils,
    when,
    Grid,
    Cache,
    SingleSort,
    Memory,
    patientInsuranceService
) {
    return declare([_Component,_TemplatedMixin,_WidgetsInTemplateMixin,_EditorViewMixin], {
        templateString: template,

        lockCheckEnabled: true,
        objectType: "Patient.Insurance.Utility", 

        // properties
        title: "General",
        helpPath: "/help/PatientInsurance.html",
        mode: "view",
        data: null,

        // sub-widgets
        policyGrid: null,
        policySequenceGrid: null,
        editor: null,
        sequenceView: null,
        policyView: null,

        /// internals
        policyStore: null,
        ndocNum: null,

        constructor: function() {
            this.addInvalidatingProperties(["data"]);
            this.policyStore = new Memory();
        },

        getTitle: function() {
            return (this.props.mode && this.navigationActive === "edit" ? "Modify " : "") + this.title;
        },

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                mode: "view",
                ndocNum: ""
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            var createGrid = function(structure,node) {
                var grid = new Grid({
                    cacheClass: Cache,
                    store: new Memory(),
                    structure: structure,
                    autoHeight: true,
                    modules: [ SingleSort ],
                    modelExtensions: [
                        "gridx/core/model/extensions/FormatSort"
                    ]
                });
                node.appendChild(grid.domNode);
                grid.startup();
                return grid;
            }

            var policySequenceStructure = [
                { id: "effectiveDate", field: "effectiveDate", name: "Effective From", formatter: utils.formatters.date("effectiveDate") },
                { id: "primaryPolicyId", field: "primaryPolicy", name: "Primary Policy", formatter: utils.formatters.patientPolicy("primaryPolicyId",t.policyStore), sortFormatted: true },
                { id: "secondaryPolicyId", field: "secondaryPolicy", name: "Secondary Policy", formatter: utils.formatters.patientPolicy("secondaryPolicyId",t.policyStore), sortFormatted: true },
                { id: "tertiaryPolicyId", field: "tertiaryPolicy", name: "Tertiary Policy", formatter: utils.formatters.patientPolicy("tertiaryPolicyId",t.policyStore), sortFormatted: true }
            ];
            t.policySequenceGrid = createGrid(policySequenceStructure,t.policySequenceGridContainer);

            var policyStructure = [
                { id: "insurance", field: "insurance", name: "Insurance" },
                { id: "policyNum", field: "policyNum", name: "Policy Number" },
                { id: "verifyDate", field: "verifyDate", name: "Verify Date", formatter: utils.formatters.date("verifyDate") }    
            ];
            t.policyGrid = createGrid(policyStructure,t.policyGridContainer);

            t.addSubComponent(t.editor);
        },

        getPageProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                content: t.domNode,
                actions : (t.props.mode === "view" ? [
                    { label: "Modify",          onClick: function() { t.set("props",{"mode":"edit"}); } },
                    { label: "Add Policy",      onClick: function() { t.loadSubView("policyView"); } },
                    { label: "Add Sequence",    onClick: function() { t.loadSubView("sequenceView"); } }
                ] : [
                    { label: "Cancel",          onClick: function() { t.okToClose(function() {t.set("props",{"mode":"view"});}) } },
                    { label: "Save",            onClick: function() { t.startSave(); },             disabled: !t.editor.modified },
                    { label: "Reset",           onClick: function() { t.invalidateRendering() },    disabled: !t.editor.modified }
                ]),
                accountSummaryShown: true,
                printButtonShown: true,
                viewMenuShown: (t.props.mode === "view"),
                report: {
                    reportClass: "Patient.Insurance.Reports",
                    getFilters: lang.hitch(t,"getReportFilters")
                }
            });
        },
 
        refreshRendering: function() {
            var t = this;
            if (t.props) {
                t.props.objectId = t.props.ndocNum;
            }
            if (t.isShown) {
                t.set("ndocNum",t.props.ndocNum);

                t.editor.set("props",{
                    disabled: t.props.mode === "view",
                    value: t.data
                });
            }
            t.inherited(arguments);
        },

        onShow: function() {
            var t = this;
            if (!t.firstShow) {
                t.resize();
            } else {
                // Preload views
                t.addSubView("policyView","thornberry/app/Component/PatientInsurance/AddEditPolicyView",function() {
                    return { ndocNum: this.props.ndocNum, objectId: "", mode: "edit" };
                },true);
                t.addSubView("sequenceView","thornberry/app/Component/PatientInsurance/AddEditSequenceView",function() {
                    return { ndocNum: this.props.ndocNum, objectId: "" };
                },true);
            }
        },

        save: function() {
            var t = this;
            var data = t.editor.get("value");
            return patientInsuranceService.saveGeneral(t.props.ndocNum,data).then(function(success) {
                t.set("data",lang.mixin(t.data,data));
                t.set("props",{"mode":"view"});
            });
        },

        updateGeneral: function(data) {
            var t = this;
            data = data.length ? data[0] : {};
            t.set("data",data);
        },

        _setNdocNumAttr: function(value) {
            var t = this;
            if (t.ndocNum !== value && value) {
                t.ndocNum = value;

                t.addReadyPromise(patientInsuranceService.getGeneral(value).then(null,lang.hitch(t,"quitOnReject")));
                t.addReadyPromise(patientInsuranceService.getPolicies(value).then(null,lang.hitch(t,"quitOnReject")));
                t.addReadyPromise(patientInsuranceService.getPolicySequences(value).then(null,lang.hitch(t,"quitOnReject")));

                t.setHandle("watchGeneral",patientInsuranceService.watchGeneral(value,lang.hitch(t,"updateGeneral")));
                t.setHandle("watchPolicySequences",patientInsuranceService.watchPolicySequences(value,lang.hitch(t,"updatePolicySequence")));
                t.setHandle("watchPolicies",patientInsuranceService.watchPolicies(value,lang.hitch(t,"updatePolicies")));
            }
        },
        
        updatePolicies: function(data) {
            var t = this;
            t.policyGrid.setStore(new Memory({
                data: arrayUtils.filter(data,function(item) {return !item.isErrored;})
            }));
            t.policyGrid.model.clearCache();
            t.policyGrid.body.refresh();

            t.policyStore.setData(data);
            t.policySequenceGrid.model.clearCache();
            t.policySequenceGrid.body.refresh();
        },

        updatePolicySequence: function(data) {
            var t = this;
            t.policySequenceGrid.setStore(new Memory({
                data: data
            }));
            t.policySequenceGrid.model.clearCache();
            t.policySequenceGrid.body.refresh();
        },

        onWindowResize: function() {
            this.resize();
        },

        resize: function() {
            var t = this;
            t.policySequenceGrid.resize();
            t.policyGrid.resize();
        },

        okToClose: function(onComplete) {
            when(this.onBeforeClose(),function(done) { 
                if (done) { 
                    onComplete();
                } 
            });
        },

        onLockFailure: function(deferred) {
            // default implementation - if we were in view mode before switching to edit, then stay in view mode, otherwise pop the view
            var t = this;
            t.props.mode = "view";
            if (deferred) {
                deferred.resolve();
            }
        },

        _openPolicySequence: function() {
            var t = this;
            t.okToClose(function() {
                t.set("props",{"mode":"view"});
                t.unlock();
                t.props.onSwitchView(1);
            });
        },
        
        _openPatientPolicies: function() {
            var t = this;
            t.okToClose(function() {
                t.set("props",{"mode":"view"});
                t.unlock();
                t.props.onSwitchView(2);
            });
        },

        getReportFilters: function(presentation,report) {
            var t = this;
            var filters =  [
                {
                    column: "NDocNumber",
                    displayName: "Patient",
                    value: [ t.props.ndocNum ],
                    displayValue: t.data.accountNum
                }
            ];
            if (report.className === "Report.Definition.PatientInsuranceWithHistory") {
                filters.push({
                    column: "id",
                    value: arrayUtils.map(t.policyGrid.rows(),function(row) {return row.id;}),
                    hidden: true
                });
            } else if (report.className === "Report.Definition.PolicySequenceWithHistory") {
                filters.push({
                    column: "id",
                    value: arrayUtils.map(t.policySequenceGrid.rows(),function(row) {return row.id;}),
                    hidden: true
                });
            }
            return filters;
        }
    });
});