define([
    "dojo/_base/declare",
    "thornberry/app/Component/_Component",
    "thornberry/app/Component/_ViewMixin",
    "thornberry/report/ReportMasterController",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom",
    "dojo/dom-style",
    "dojo/dom-construct",
    "dojo/query",
    "dojo/aspect",
    "dojo/Deferred",
    "dojo/promise/all",
    "thornberry/serverRequest"
],function(
    declare,
    _Component,
    _ViewMixin,
    ReportMasterController,
    lang,
    arrayUtils,
    dom,
    domStyle,
    domConstruct,
    query,
    aspect,
    Deferred,
    all,
    serverRequest
) {
    var SubView = declare([_Component,_ViewMixin],{
        content: null,
        controller: null,
        onApply: null,
        printable: false,
        searchable: false,
        presetControlContainer: null,
        showPresetControl: false,

        getPageProps: function() {
            var t = this;
            return {
                content: t.content,
                actions : [
                    { label: "Cancel", onClick: function() { t.onCancel(); t.props.onCloseView();} },
                    [ { label: "Apply", onClick: function() { t.onApply(); t.props.onCloseView();} } ]
                ],
                helpButtonShown:    true,
                printButtonShown:   t.printable,

                onClickHelp: function() { t.controller.showHelp(); },
                onClickPrint: function() { t.controller.reportManager.show(); }
            };
        },

        refreshRendering: function() {
            var t = this;
            t.addReadyPromise(all(t.controller.loadDeferredArray));
            t.inherited(arguments);
        },

        onShow: function() {
            var t = this;
            if (t.showPresetControl) {
                domStyle.set(t.presetControlContainer,"display","");
            }
        },

        onHide: function() {
            var t = this;
            domStyle.set(t.presetControlContainer,"display","none");
        }
    });

    return declare([_Component,_ViewMixin],{
        reportClass: "",

        controller: null,
        summarizable: false,
        printable: false,

        presetControlContainer: null,
        showPresetControl: true,
        defaultToFilterView: true, // If the report has filters, then show the filter pane first when opening up if true

        filters: null,

        controllerProps: null,

        getTitle: function() {
            var t = this;
            return (t.title ? t.title : t.controller.report.displayName);
        },

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                title: "",
                reportClass: "",
                selectedIds: [],
                filters: null
            });
        },

        getPageProps: function() {
            var t = this;
            return {
                content: t.controller.gridView.domNode,
                actions : [
                    { label: "Refresh", onClick: function() {t.controller.gridView.updateGridData(true);} }
                ],
                helpButtonShown:    true,
                filterButtonShown:  t.controller.report.filterColumns !== "",
                optionsButtonShown:  true,
                sumButtonShown: t.summarizable,
                printButtonShown: t.printable,
                searchButtonShown: t.searchable,

                viewMenuShown: true,

                onClickHelp: function() { t.controller.showHelp(); },
                onClickFilter: function() { t.props.onPushView(t.filterView); },
                onClickOptions: function() { t.props.onPushView(t.optionView); },
                onClickSum: function() { t.props.onPushView(t.sumView); },
                onClickPrint: function() { t.controller.reportManager.show(); },
                onSearch: function(value) { t.controller.set("search",value); }
            };
        },

        refreshRendering: function() {
            var t = this;
            //if (t.isShown) {
                t.set("reportClass",t.props.reportClass);
            //}
            t.set("filters",t.props.filters);
            t.inherited(arguments);
        },

        _setReportClassAttr: function(reportClass) {
            var t = this;
            if (t.reportClass === "" && reportClass) {
                t.reportClass = reportClass;
                // Only load once

                var deferred = new Deferred();
                var controllerLoadDeferred = new Deferred();
                new ReportMasterController.ControllerItem({
                    reportClass:reportClass,
                    props: lang.mixin({
                        postCreate: function() {
                            var t = this;
                            t.loadDeferredArray.push(controllerLoadDeferred.promise);
                            t.inherited("postCreate",arguments);
                        }
                    },t.controllerProps)
                }).load().then(function(controller) {
                    t.onLoadController(controller,controllerLoadDeferred).then(function() {
                        t.set("filters",t.props.filters);
                        deferred.resolve();
                    });
                });
                t.addReadyPromise(deferred.promise);
            }
        },

        onLoadController: function(controller,controllerLoadDeferred) {
            var t = this;

            t.controller = controller;
            var gridView = controller.gridView;
            dom.byId("hiddenContentNode").appendChild(controller.domNode);
            controller.startup();

            controller.popView = function() {};
            if (gridView) {
                domStyle.set(gridView.domNode,"padding","0px");

                gridView.grid.connect(gridView.grid,"onCellDblClick",function(evt) {
                    t.onRowDoubleClick(evt.rowId);
                });
            }

            var actionPaneNodes = query(".action-pane");
            if (actionPaneNodes && actionPaneNodes.length > 0) {
                t.presetControlContainer = domConstruct.toDom("<div style='position:relative;margin:auto;width:350px;display:none'></div>");
                actionPaneNodes[0].appendChild(t.presetControlContainer);

                t.presetControlContainer.appendChild(t.controller.presetDropDownButton.domNode);
                t.presetControlContainer.appendChild(t.controller.presetSaveButton.domNode);
            }

            t.printable = t.controller.report.printable();
            t.searchable = t.controller.report.searchable;

            if (controller.report.filterColumns !== "") {
                t.filterView = new SubView({
                    title: "Filters",
                    controller: controller,
                    content: controller.filterView.domNode,
                    onCancel: lang.hitch(t.controller,"onCancelFilters"),
                    onApply: function() {
                        t.filters = null;
                        t.props.filters = null;
                        t.controller.onApplyFilters();
                    },
                    printable: t.printable,
                    presetControlContainer: t.presetControlContainer,
                    showPresetControl: t.showPresetControl
                });
                if (t.defaultToFilterView) {
                    t.props.onPushView(t.filterView);
                }
            }

            t.optionView = new SubView({
                title: "Options",
                controller: controller,
                content: controller.optionView.domNode,
                onCancel: lang.hitch(t.controller,"onCancelOptions"),
                onApply: lang.hitch(t.controller,"onApplyOptions"),
                presetControlContainer: t.presetControlContainer,
                showPresetControl: t.showPresetControl
            });

            if (controller.report.isSummarizable && controller.report.summarizableColumns !== "") {
                t.summarizable = true;
                t.sumView = new SubView({
                    title: "Summarization",
                    controller: controller,
                    content: controller.summarizationView.domNode,
                    onCancel: lang.hitch(t.controller,"onCancelSummarization"),
                    onApply: lang.hitch(t.controller,"onApplySummarization"),
                    presetControlContainer: t.presetControlContainer,
                    showPresetControl: t.showPresetControl
                });
            }

            gridView.updateGridData = function(force) {
                // Summary:
                //      This method should be called whenever the user makes changes that affect what data displays in the grid. It gathers
                //      all the context information and passes it to the report class's implementation of Report.Definition.Abstract:GenerateData()
                //      and then passes the results to the GridView
                if ((gridView.controller.modified.any || force) && t.isShown) {
                    gridView.grid.vScroller.scrollToRow(0,1);
    
                    if (gridView.controller.modified.options) {
                        gridView.setColumns(gridView.controller.columnPane.get("value"));
                    }
                    if (gridView.controller.modified.any) {
                        gridView.resetPagination();
                    }

                    var context = gridView.controller.getContext("Interactive",gridView.controller.report);
                    context.pageStart = gridView.get("dataPageStart");
                    context.pageSize = gridView.get("dataPageSize");
    
                    var loadPromise = serverRequest(gridView.controller.report.className,"GenerateData",context,gridView.controller.report.async).then(function(results) {
                        gridView.setResults(results);
                        gridView.controller.clearModified();
                    },function(error) {
                        gridView.controller.showMessage(error);
                    });
                    if (t.isShown) {
                        t.props.onSetLoadingPromise(loadPromise);
                    }
                    t.onRefreshData();
                }
            }

            if (controller.report.selectableRows) {
                gridView.grid.connect(gridView.grid.select.row,"onSelectionChange",lang.hitch(t,"_onChangeSelection"));
            }

            t.own(aspect.after(controller,"getContext",function(context) {
                return t.onGetContext(context);
            }));
            controllerLoadDeferred.resolve();
            return all(controller.loadDeferredArray);
        },

        _setFiltersAttr: function(filters) {
            var t = this;
            if (filters && t.controller) {
                if (t.filters !== filters) {
                    // Clear any previously set values
                    arrayUtils.forEach(t.controller.filterEditor.fields,function(field) {
                        field.deserialize(t.controller.filterDefaults[field.name]);
                    });
                }
                var currentPreset = t.controller.presetManager.currentPreset;
                arrayUtils.forEach(t.controller.filterEditor.fields,function(field) {
                    if (field.name in filters) {
                        field.deserialize(filters[field.name]);
                    }
                    currentPreset.filters[field.name] = field.serialize();
                });
                if (t.filters !== filters) {
                   // t.controller.setModified("filters");
                   t.controller.modified.filters = true;
                   t.controller.modified.any = true;
                }
            }
            t.filters = filters;
        },

        _onChangeSelection: function() {
            this.onChangeSelection(this.getSelected(this.controller.gridView.grid));
        },

        onChangeSelection: function(ids) {
            this.set("props",{selectedIds:ids})
        },

        onRowDoubleClick: function(id) {},

        getSelected: function(grid) {
            var selectedIds = [];
            var rows = grid.rows();
            for (var i = 0; i < rows.length; i++) {
                if (grid.select.row.isSelected(rows[i].id)) {
                    selectedIds.push(rows[i].id);
                }
            }
            return selectedIds;
        },

        getGridData: function(id) {
            return this.controller.gridView.grid.store.get(id);
        },

        onShow: function() {
            var t = this;
            if (t.showPresetControl) {
                domStyle.set(t.presetControlContainer,"display","");
            }
            t.controller.gridView.updateGridData();
        },

        onHide: function() {
            var t = this;
            t.defer(function() {
                domStyle.set(t.presetControlContainer,"display","none");
            });
        },

        onWindowResize: function() {
            this.props.onResizeContainer();
        },

        refreshData: function() {
            var t = this;
            if (t.controller) {
                if (t.isShown) {
                    t.controller.gridView.updateGridData(true);
                } else {
                    t.controller.modified.any = true;
                }
            }
        },

        onRefreshData: function() {},

        onGetContext: function(context) {
            return context;
        }
    });
});