define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/query",
    "dojo/dom-attr",
    "thornberry/app/utils"
],function(
    declare,
    lang,
    arrayUtils,
    query,
    domAttr,
    utils
) {
    return declare(null,{
        // use data-form-name="{property}" attribute on all nodes to be bound to a property in this.value

        _dataSelectorNodeMap: null,

        constructor: function() {
            this._dataSelectorNodeMap = {};
        },

        postCreate: function() {
            var t = this;
            query("*[data-form-name]",t.domNode).forEach(function(node) {
                var selector = domAttr.get(node,"data-form-name");
                t._addDataNode(selector,node);
            });
            t.inherited(arguments);
        },

        _addDataNode: function(selector,node) {
            var t = this;
            t.inherited(arguments);
            if (!(selector in t._dataSelectorNodeMap)) {
                t._dataSelectorNodeMap[selector] = [];
            }
            t._dataSelectorNodeMap[selector].push(node);
        },

        _setValueAttr: function(value) {
            var t = this;
            t.inherited(arguments);
            for (selector in t._dataSelectorNodeMap) {
                var pieces = selector.toString().split("[");
                var attribute = pieces[0];
                var index = (pieces.length > 1 ? pieces[1].toString().split("]")[0] : null);
                var dataValue = lang.getObject(attribute,false,value);
                dataValue = (typeof dataValue === "undefined" || dataValue === null ? "" : dataValue);
                if (dataValue && index) {
                    if (dataValue.length > index) {
                        dataValue = dataValue[index];
                    } else {
                        dataValue = "";
                    }
                }

                var nodes = t._dataSelectorNodeMap[selector];
                arrayUtils.forEach(nodes,function(node) {
                    try {
                        var format      = domAttr.get(node,"data-form-format");
                        var formatter   = utils.formatters[format];
                        node.innerText = (formatter ? formatter("value")({value:dataValue}) : dataValue);
                    } catch (e) {}
                });
            }
        }
    });
});