define([
    "dojo/_base/declare",
    "thornberry/object/TableManager",
    "dojo/_base/array",
    "dojo/store/Memory",
    "dojo/dom-construct",
    "dojo/promise/all"
],function(
    declare,
    tableManager,
    arrayUtils,
    Memory,
    domConstruct,
    all
) {
    return declare(null, {
        tableManager: tableManager,

        _tableLoadPromise: null,
        _tableFields: null,
        _tableWaiting: false,
        _tables: null, // a store of tables managed by the component (not the table manager)

        tableFilters: null,
        addTableFilter: function(names,filter) {
            // tableFilter = function(name,tableItem,value) {return include;} return true to include the table item, or false otherwise
            // Only a single filter function may be set per field. If multiple are added, only the first is called
            this._addToNameFunctionIndex(names,filter,this.tableFilters);
        },

        constructor: function() {
            this._tableFields = [];
            this.tableFilters = {};
            this._tables = new Memory();
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);

            var loadPromiseArray = [];
            arrayUtils.forEach(t.fields,function(field) {
                if (field.tableName) {
                    if (field.managedTable) {
                        loadPromiseArray.push(tableManager.get(field.tableName,true));
                    }
                    t._tableFields.push(field);
                }
            });
            tableManager.load();
            t._tableLoadPromise = all(loadPromiseArray);
        },

        setTable: function(name,data /* array */,updateFields) {
            var t = this;
            var store = new Memory({ data: data });
            t._tables.put({ id: name, store: store });
            if (updateFields) {
                arrayUtils.forEach(t._tableFields,function(field) {
                    if (field.tableName === name) {
                        t.refreshTableField(field.name);
                    }
                });
            }
        },

        getTable: function(tableName) {
            return this._tables.get(tableName).store;
        },

        setTableValue: function(name,value,triggerChanges,filter) {
            
        },

        refreshTableField: function(name,value) {
            var t = this;
            var field = t.fieldMap[name][0];
            var value = (typeof value === "undefined" ? field.get("value") : value);
            var filter = (t.tableFilters[name] ? t.tableFilters[name][0] : null);
            if (!filter) {
                filter = (t.tableFilters[""] ? t.tableFilters[""][0] : null);
            }
            t._applyTable(field,value,filter,t.getBoundLabel(name));
        },

        refreshRendering: function() {
            var t = this;
            t.inherited(arguments);
            if (t._tableLoadPromise !== null && !t._tableLoadPromise.isResolved()) {
                t.addReadyPromise(t._tableLoadPromise);
            }
        },
        
        _setValueAttr: function(value) {
            var t = this;
            var needsRefresh = t._needsRefresh(value);
            t.inherited(arguments);
            if (needsRefresh) {
                t._applyTables(true);
            }
        },

        _applyTables: function(init) {
            var t = this;
            if (t._tableLoadPromise !== null) {
                if (t._tableLoadPromise.isResolved()) {
                    t._tableWaiting = false;
                    var last = t.getHistory();
                    arrayUtils.forEach(t._tableFields,function(field) {
                        var name = field.name;
                        var filter = (t.tableFilters[name] ? t.tableFilters[name][0] : null);
                        if (!filter) {
                            filter = (t.tableFilters[""] ? t.tableFilters[""][0] : null);
                        }
                        t._applyTable(field,last[name],filter,t.getBoundLabel(field.name),init);
                    });
                } else if (!t._tableWaiting) {
                    t._tableWaiting = true;
                    t._tableLoadPromise.then(function() {
                        t._applyTables(init);
                        t.invalidate("fieldState");
                    });
                }
            }
        },

        getLabel: function(name,item) {
            return item.name || item.Name;
        },

        getBoundLabel: function(name) {
            var t = this;
            return function(item) {
                return t.getLabel(name,item);
            };
        },

        _applyTable: function(field,value,filter,getLabel,init) {
            var t = this;
            var table;
            if (field.managedTable) {
                table = tableManager.get(field.tableName);
            } else {
                var item = t._tables.get(field.tableName);
                if (item) {
                    table = item.store;
                }
            }
            table = table || new Memory();
            if (field.declaredClass !== "dojox.form.CheckedMultiSelect" || field.getOptions().length === 0) { // CheckedMultiSelect can only be setup once
                var filteredArray = [];
                if (filter) {
                    filteredArray = arrayUtils.filter(table.data,function(item) {
                        return filter(field.name,item,value) || (init && item.id == value);
                    });
                } else {
                    filteredArray = table.data.slice();
                }

                if (field.declaredClass === "dijit.form.Select") {
                    filteredArray.unshift({ id: "", name: "(Select)" });
                    var filteredStore = new Memory({ data: filteredArray, getLabel: getLabel });
                    field.setStore(filteredStore);
                } else if (field.declaredClass === "dijit.form.FilteringSelect") {
                    var filteredStore = new Memory({ data: filteredArray, getLabel: getLabel });
                    field.set("store",filteredStore);
                } else if (field.declaredClass === "dijit.form.MultiSelect") {

                } else if (field.declaredClass === "dojox.form.CheckedMultiSelect") {
                    arrayUtils.forEach(filteredArray,function(item) {
                        field.addOption(domConstruct.create("option",{label:escapeText(getLabel(item)),value:item.id}));
                    });
                }
            }

            field._onChangeActive = false;
            field.set("value",value);
            field._onChangeActive = true;
        }
    });
});