define([
    "dojo/store/Memory",
    "dojo/_base/array",
    "thornberry/serverRequest",
    "dojo/dom",
    "dojo/on",
    "thornberry/app/utils"
],function(
    Memory,
    arrayUtils,
    serverRequest,
    dom,
    on,
    utils
) {
    // This singleton serves as an intermediary between a WebLink page and a dojo component or components performing file uploads
    
    return {
        fileUploadInputId: "", // Set by the WebLink page to the file input element's ID. It can then be used as the "for" attribute on labels in widgets to trigger actions
        fileUploadSubmitId: "", // Set by the WebLink page to the form's submit button's ID. It can then be used as the "for" attribute on labels in widgets to trigger actions
        uploadFormNode: null, // Set by the WebLink page to the form used to upload the file. It is used to reset the form when an upload is finished

        _nextSubId: 1,
        _subscriptions: new Memory(),
        _filePath: "",

        init: function(fileUploadInputId,fileUploadSubmitId,uploadFormId) {
            var t = this;
            t.fileUploadInputId = fileUploadInputId;
            t.fileUploadSubmitId = fileUploadSubmitId;
            t.uploadFormNode = dom.byId(uploadFormId);

            on(dom.byId(t.fileUploadInputId),"change",function() {
                t.selectFile(dom.byId(t.fileUploadInputId).value);
            });
        },

        reset: function() {
            this.uploadFormNode.reset();
        },
        
        addSubscriber: function(subscriber,cacheClass,cacheMethod) {
            /*
                subscribe to receive notification of events
                - onSelectFile(filePath)
                - onUpload(promise)
                - onUploadFailure(message);
            */

            var t = this;
            if (subscriber) {
                var id = t._lastSubId++;                                                                                                                                        
                t._subscriptions.put({ id: t._lastSubId++, subscriber: subscriber, cacheClass: cacheClass, cacheMethod: cacheMethod });

                // return a handle to remove the subscription
                return { 
                    remove: function() { 
                        t._subscriptions.remove(id); 
                    }
                };
            }
        },

        selectFile: function(filePath) {
            this._filePath = filePath;
            this._publish("onSelectFile",filePath);
        },

        uploadComplete: function(status) {
            var t = this;
            if (status == 1) {
                arrayUtils.forEach(t._subscriptions.data,function(subscription) {
                    if (subscription.cacheMethod !== "" && subscription.cacheClass !== "") {
                        var options = {
                            filePath: t._filePath,
                            cacheClass: subscription.cacheClass,
                            cacheMethod: subscription.cacheMethod
                        }
                        var promise = serverRequest("UI.Utility","OnUploadFile",options);
                        t._publishOne(subscription.subscriber,"onUpload",promise);
                    }
                });
                //t.uploadFormNode.reset();
                t._filePath = "";
            } else {
                this._publish("onUploadFailure","The file failed to upload.");
            }
        },

        _publish: function(method,arg1,arg2) {
            var t = this;
            arrayUtils.forEach(t._subscriptions.data,function(subscription) {
                t._publishOne(subscription.subscriber,method,arg1,arg2);
            });
        },

        _publishOne: function(subscriber,method,arg1,arg2) {
            if (subscriber && subscriber[method]) {
                subscriber[method](arg1,arg2);
            }
        }
    }
});