define([
	"dojo/_base/declare",
	"thornberry/layout/MasterController",
	"dojo/Deferred",
	"thornberry/serverRequest",
	"thornberry/report/Report",
	"thornberry/report/Controller",
	"thornberry/report/SummarizableController",
	"thornberry/report/CrossTabController",
	"thornberry/report/BasicController",
	"thornberry/PresetManager",
	"dojo/dom-construct",
	"dojo/on",
	"dojo/_base/lang",
	"thornberry/UserSettings",
	"dojo/store/Memory",
	"dojo/aspect",
	"dojo/dom-style",
	"dojo/_base/array",
	"dojo/query",
	"dojo/promise/all"
],function(declare,MasterController,Deferred,serverRequest,Report,Controller,SummarizableController,CrossTabController,BasicController,PresetManager,domConstruct,on,lang,UserSettings,Memory,aspect,domStyle,arrayUtils,query,all) {
	var ControllerItem = MasterController.ControllerItem;
	var ReportControllerItem = declare(ControllerItem,{
		reportClass: "",
		
		props: null,
		
		_loadDeferred: null,
		
		load: function() {
			var t = this;
			if (t._loadDeferred) {
				return t._loadDeferred;
			} else {
				t._loadDeferred = new Deferred();

				serverRequest("Report.ZenReport.Utility","LoadReportDefinition",t.reportClass).then(function(reportData) {
					var report = new Report(reportData.reportDefinition);
					var presetManager = t.constructPresetManager(reportData);
					
					var ControllerClass = Controller;
                    if (report.isBasic) {
                        ControllerClass = BasicController;
                    } else if (report.isSummarizable && report.summarizableColumns !== "") {
                        ControllerClass = SummarizableController;
                    } else if (report.isCrossTab) {
                        ControllerClass = CrossTabController;
					}
					var userSettings = new UserSettings({
						userId: userId,
						category: reportData.permissionId,
						store: new Memory({data:reportData.userSettings.children})
					});

                    var controller = new ControllerClass(lang.mixin({
						report: report,
						userSettings: userSettings,
                        presetManager: presetManager,
                        showExit: false,
                        generalUserSettings: generalUserSettings,
                        deviceAppId: deviceAppId,
                        filterOptions: reportData.filterOptions,
						filterDefaults: reportData.filterDefaults,
						showGridOnOpen: true,
						_setShowExitAttr: function() {}
					},t.props));
					
					t._loadDeferred.resolve(controller);
				},function(error) {
					t._loadDeferred.reject(error);
				});
			}
			return t._loadDeferred.promise;
		},
		
		constructPresetManager: function(reportData) {
			return new PresetManager({
				presetData: reportData.reportDefinition.presets,
				userId: userId,
				providerClass: reportData.reportDefinition.className
			});
		}
	});
	
	var ReportMasterController = declare(MasterController, {
		_filterStore: null,

		postMixInProperties: function() {
			var t = this;
			t.inherited(arguments);

			t._filterStore = new Memory();
		},

		_setControllerAttr: function(controller) {
			var t = this;

			if (t.controlller === controller) {
				return;
			}

			if (t.controller) {
				arrayUtils.forEach(t.controller.filterWidgets,function(filterWidget) {
					var filterName = filterWidget.name;
					var filterValue = filterWidget.serialize();
					if (filterName === "NDocNumber" || filterName === "ClinicianInternalCalendar") {
						// Block should be for any search types
						filterValue = [];
						query("option", filterWidget.selectedResults.domNode).forEach(function (node) {
							filterValue.push(lang.clone(node));
						});
					}
					t._filterStore.put({id:filterName,value:filterValue});
				});
			}

			t.inherited(arguments);
			
			if (t.controller.presetDropDownButton) {
				t._currentControllerHandles.push(aspect.after(t.controller,"refresh",function() {
					domStyle.set(t.controller.presetDropDownButton.domNode,"display",(t.titlePane.breadcrumbs.length > 1 ? "none" : ""));
					if (t.titlePane.breadcrumbs.length > 1) {
						domStyle.set(t.controller.presetSaveButton.domNode,"display","none");
					}
				},true));
			}

			all(t.controller.loadDeferredArray).then(function() {
				var changedFilters = false;
				arrayUtils.forEach(t._filterStore.data,function(filterItem) {
					arrayUtils.some(t.controller.filterWidgets,function(widget) {
						var found = false;
						if (widget.name === filterItem.id) {
							if (filterItem.id === "NDocNumber" || filterItem.id === "ClinicianInternalCalendar") {
								// Should actually be for any search filter
								t.controller.filterEditor[filterItem.id+"Field"]._removeAll();
								arrayUtils.forEach(filterItem.value,function(option) {
									widget.selectedResults.domNode.appendChild(option);
								});
							} else {
								t.controller.filterEditor[filterItem.id+"Field"].deserialize(filterItem.value);
							}
							changedFilters = true;
							found = true;
						}
						return found;
					});
				});
				if (changedFilters) {
					t.defer(function() {
						t.controller.onApplyFilters();
					});
				}
			});
		}
	});
	ReportMasterController.ControllerItem = ReportControllerItem;
	return ReportMasterController;
});